@@ -262,7 +262,7 @@ test_api(void)
     int			n;
     int			numpmid = MAXNAMES;
     char		**names;
-    pmHighResResult	*resp;
+    pmResult		*resp;
     pmDesc		desc;
 
     _op++;
@@ -321,15 +321,15 @@ test_api(void)
 	struct timespec	when, delta = { 1, 0 };
 
 	_op++;
-	if ((n = pmGetHighResArchiveEnd(&when)) < 0) {
+	if ((n = pmGetArchiveEnd(&when)) < 0) {
 	    _err++;
-	    printf("pmGetHighResArchiveEnd: %s\n", pmErrStr(n));
+	    printf("pmGetArchiveEnd: %s\n", pmErrStr(n));
 	}
 
 	_op++;
-	if ((n = pmSetModeHighRes(PM_MODE_BACK, &when, &delta)) < 0) {
+	if ((n = pmSetMode(PM_MODE_BACK, &when, &delta)) < 0) {
 	    _err++;
-	    printf("pmSetModeHighRes(PM_MODE_BACK): %s\n", pmErrStr(n));
+	    printf("pmSetMode(PM_MODE_BACK): %s\n", pmErrStr(n));
 	}
     }
 
@@ -447,23 +447,23 @@ test_api(void)
 	    if (midlist[i] == PM_ID_NULL)
 		continue; 
 	    _op++;
-	    if ((n = pmSetModeHighRes(PM_MODE_FORW, &when, NULL)) < 0) {
+	    if ((n = pmSetMode(PM_MODE_FORW, &when, NULL)) < 0) {
 		_err++;
-		printf("pmSetModeHighRes(PM_MODE_FORW): %s\n", pmErrStr(n));
+		printf("pmSetMode(PM_MODE_FORW): %s\n", pmErrStr(n));
 	    }
 	    else {
 		_op++;
                 if (vflag)
                     printf("Fetch of %s:\n", namelist[i]);
-		if ((n = pmFetchHighRes(1, &midlist[i], &resp)) < 0) {
+		if ((n = pmFetch(1, &midlist[i], &resp)) < 0) {
 		    _err++;
-		    printf("Archive pmFetchHighRes: %s\n", pmErrStr(n));
+		    printf("Archive pmFetch: %s\n", pmErrStr(n));
 		}
 		else {
 		    if (vflag)
-			__pmDumpHighResResult(stdout, resp);
+			__pmDumpResult(stdout, resp);
 		    _op++;
-		    pmFreeHighResResult(resp);
+		    pmFreeResult(resp);
 		}
 	    }
   	}/*for*/
@@ -471,23 +471,23 @@ test_api(void)
 
     else if (context_type == PM_CONTEXT_HOST) {
 	_op++;
-	if ((n = pmSetModeHighRes(PM_MODE_LIVE, NULL, NULL)) < 0) {
+	if ((n = pmSetMode(PM_MODE_LIVE, NULL, NULL)) < 0) {
 	    _err++;
-	    printf("pmSetModeHighRes(PM_MODE_LIVE): %s\n", pmErrStr(n));
+	    printf("pmSetMode(PM_MODE_LIVE): %s\n", pmErrStr(n));
 	}
 	else {
 	    _op++;
-	    if ((n = pmFetchHighRes(numpmid, midlist, &resp)) < 0) {
+	    if ((n = pmFetch(numpmid, midlist, &resp)) < 0) {
 		_err++;
-		printf("real-time pmFetchHighRes: %s\n", pmErrStr(n));
+		printf("real-time pmFetch: %s\n", pmErrStr(n));
 	    }
 	    else {
 		if (vflag) {
 		    printf("\nReal-time result ...\n");
-		    __pmDumpHighResResult(stdout, resp);
+		    __pmDumpResult(stdout, resp);
 		}
 		_op++;
-		pmFreeHighResResult(resp);
+		pmFreeResult(resp);
 	    }
 	}
     }
