@@ -1,6 +1,6 @@
 #!/bin/sh
 #
-# Copyright (c) 2018 Red Hat.
+# Copyright (c) 2018,2020 Red Hat.
 #
 # This program is free software; you can redistribute it and/or modify it
 # under the terms of the GNU General Public License as published by the
@@ -18,10 +18,8 @@ PCP_KUBECTL_PROG=${PCP_KUBECTL_PROG-'kubectl'}
 which $PCP_KUBECTL_PROG >/dev/null 2>/dev/null || exit 0
 [ -e "$PCP_SYSCONF_DIR/discover/pcp-kube-pods.disabled" ] && exit 0
 
-args="--show-all -o jsonpath={.items[*].status.podIP}"
+args="-o jsonpath={.items[*].status.podIP}"
 file="$PCP_SYSCONF_DIR/discover/pcp-kube-pods.conf" 
 [ -f "$file" ] && args=`cat "$file"`
-file="$PCP_SYSCONF_DIR/pmmgr/target-kubectl-pod" # back-compat
-[ -f "$file" ] && args="$args `cat "$file"`"
 
-exec $PCP_KUBECTL_PROG get pod $args
+exec $PCP_KUBECTL_PROG get pods $args
