@@ -22,8 +22,8 @@ import sys
 from datetime import datetime, timezone
 from pcp import pmapi, pmi
 from cpmapi import (PM_COUNT_ONE, PM_LABEL_CONTEXT,
-                    PM_TIME_SEC, PM_TIME_MSEC,
-                    PM_TYPE_DOUBLE, PM_TYPE_U64, PM_SEM_DISCRETE,
+                    PM_TIME_SEC, PM_TIME_MSEC, PM_SEM_DISCRETE,
+                    PM_TYPE_DOUBLE, PM_TYPE_U64, PM_TYPE_STRING,
                     PM_TEXT_PMID, PM_TEXT_ONELINE, PM_TEXT_HELP)
 
 def register_metrics(pcp) -> pmapi.pmInDom:
@@ -50,8 +50,10 @@ def register_metrics(pcp) -> pmapi.pmInDom:
         item += 1
 
     pmns = 'guidellm.'
+    add_metric(pmns + 'run_id', PM_TYPE_STRING, nounit,
+               'Unique identifier spanning the entire benchmark invocation')
     add_metric(pmns + 'duration', PM_TYPE_DOUBLE, secunits,
-               'Elapsed time in seconds for each GuideLLM run completed')
+               'Elapsed time in seconds for each GuideLLM strategy completion')
     pmns = 'guidellm.run_stats.requests_made.'
     add_metric(pmns + 'successful', PM_TYPE_U64, unitone,
                'Count of requests processed (requests-per-second as a rate)')
@@ -232,12 +234,13 @@ def register_metrics(pcp) -> pmapi.pmInDom:
 
     return indom
 
-def put_metric_values(pcp, values, instance) -> None:
+def put_metric_values(pcp, runid, values, instance) -> None:
     """ Scan through JSON data and extract values for each PCP metric;
         pmiPutValue inserts one value for one metric:inst at one time.
     """
     pmns = 'guidellm.'
     pcp.pmiPutValue(pmns + 'duration', instance, str(values['duration']))
+    pcp.pmiPutValue(pmns + 'run_id', instance, str(runid))
 
     pmns = 'guidellm.run_stats.requests_made.'
     stats = values['run_stats']['requests_made']
@@ -269,9 +272,22 @@ def put_metric_values(pcp, values, instance) -> None:
         pcp.pmiPutValue(pmns + 'variance', instance, str(stats['variance']))
 
 
-def timestamp(timestring) -> datetime:
+def timestamp(timestring: str) -> datetime:
     return datetime.fromtimestamp(timestring, timezone.utc)
 
+insts = {}    # hash instance names to instance IDs
+instids = {}  # hash instance IDs to instance names
+
+def getinstid(instance: str) -> int:
+    try:
+        return insts[instname]
+    except KeyError:
+        pass # fallthrough
+    key = hash(instance) & 0x7FFFFFFF
+    if instids.get(key) is None:
+        instids[key] = instance
+        return key
+    return getinstid('_' + instance)
 
 parser = argparse.ArgumentParser()
 parser.add_argument("-v", "--verbose", help="Enable verbose progress diagnostics",
@@ -287,7 +303,6 @@ with open(args.results) as json_data:
     except: # pylint: disable=bare-except
         print('Failed to load JSON document from:', args.results)
         sys.exit(1)
-    instid = 0
     run_id = None
 
     # check if the results are wrapped with model furnace metadata
@@ -318,20 +333,22 @@ with open(args.results) as json_data:
                     log.pmiPutLabel(PM_LABEL_CONTEXT, 0, 0, label, value)
             ids = register_metrics(log)
 
-        instid = instid + 1
         instname = benchmark['id_']
+        instid = getinstid(instname)
         if args.verbose:
             print('[%d] %s' % (instid, instname))
 
         log.pmiAddInstance(ids, instname, instid)
         log.pmiPutLabel(PM_LABEL_CONTEXT, 0, 0, 'guidellm_id', instname)
 
-        put_metric_values(log, benchmark, instname)
+        put_metric_values(log, run_id, benchmark, instname)
         log.pmiWrite(timestamp(benchmark['run_stats']['start_time']))
 
-        put_metric_values(log, benchmark, instname)
+        put_metric_values(log, run_id, benchmark, instname)
         log.pmiWrite(timestamp(benchmark['run_stats']['end_time']))
 
         log.pmiPutMark() # end of benchmark iteration
 
+    if args.verbose:
+        print("Writing archive:", args.archive)
     del log # flush the archive to persistent storage
