@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2012-2022 Red Hat.
+ * Copyright (c) 2012-2022,2025 Red Hat.
  * Copyright (c) 1997,2004 Silicon Graphics, Inc.  All Rights Reserved.
  *
  * This library is free software; you can redistribute it and/or modify it
@@ -54,10 +54,30 @@
 extern "C" {
 #endif
 
+/*
+ * Historical note:
+ *
+ * PMAPI_VERSION 1 only lasted for a short time in 1993-1998 (the
+ * initial development and the PCP 1.1 release in April 1995).
+ * PMAPI_VERSION 2 was part of the PCP 2.1 release in May 1998 and was the
+ * long-standing version that evolved over the next 25+ years largely with
+ * backwards compatibility.
+ * Changes to make PCP Y2038-safe involved moving to 64-bit precision for
+ * all timestamps (and for consistency, time intervals), and these
+ * changes could not be done in a manner that was backwards compatibible,
+ * so the PMAPI_VERSION had to move forwards.  But at some point (the
+ * reasons are lost in the mists of time), the libpcp DSO had moved from
+ * version 2 to version 3.  To align these two, the new PMAPI_VERSION is
+ * 4.
+ * Like the Travelling Wilbury's Volume 2, PCP's PMAPI_VERSION 3 never
+ * existed.
+ */
+
 #define PMAPI_VERSION_2	2	/* traditional PMAPI */
-#define PMAPI_VERSION_3	3	/* nanosec precision */
+#define PMAPI_VERSION_3	3	/* never existed */
+#define PMAPI_VERSION_4	4	/* timeval -> timespec */
 #ifndef PMAPI_VERSION
-#define PMAPI_VERSION	PMAPI_VERSION_2   /* default */
+#define PMAPI_VERSION	PMAPI_VERSION_4   /* current default */
 #endif
 
 /*
@@ -365,6 +385,8 @@ PCP_CALL extern int pmNewContext(int, const char *);
 					/* don't check V3 archive features */
 #define PM_CTXFLAG_NO_FEATURE_CHECK	(1U<<15) /* don't check features in label record */
 #define PM_CTXFLAG_METADATA_ONLY	(1U<<16) /* only open .meta file of archive */
+#define PM_CTXFLAG_LAST_VOLUME	(1U<<17) /* open archive at start of last volume */
+#define PM_CTXFLAG_STREAMING_WRITER	(1U<<18) /* streaming log over a network */
 
 /*
  * Duplicate current context -- returns handle to new one for pmUseContext()
@@ -498,23 +520,26 @@ typedef struct pmValueSet {
 #define PM_VAL_DPTR	1	/* value.pval->vbuf is it, and dynamic alloc */
 #define PM_VAL_SPTR	2	/* value.pval->vbuf is it, and static alloc */
 
-
-/* Result returned by pmFetch() */
+/*
+ * Result returned by pmFetch() and high resolution event time
+ * result from pmUnpackEventRecords()
+ */
 typedef struct pmResult {
-    struct timeval	timestamp;	/* time stamped by collector */
+    struct timespec	timestamp;	/* time stamped by collector */
     int			numpmid;	/* number of PMIDs */
     pmValueSet		*vset[1];	/* set of value sets, one per PMID */
 } pmResult;
 
 /*
- * Result returned by pmFetchHighRes() and high resolution event timer
- * result from pmUnpackHighResEventRecords()
+ * old (prior to PCP 7.0) pmResult with timeval, not timespec
+ * ... still needed as alias for pmdaResult and qmcResult and
+ * old low resolution event time result for packing/unpacking routines
  */
-typedef struct pmHighResResult {
-    struct timespec	timestamp;	/* time stamped by collector */
+typedef struct pmResult_v2 {
+    struct timeval	timestamp;	/* time stamped by collector */
     int			numpmid;	/* number of PMIDs */
     pmValueSet		*vset[1];	/* set of value sets, one per PMID */
-} pmHighResResult;
+} pmResult_v2;
 
 /* Generic Union for Value-Type conversions */
 typedef union {
@@ -535,12 +560,9 @@ typedef union {
  *
  * The value sets returned are in the same order as the metrics argument,
  * and the number of value sets returned is guaranteed to be the same as
- * the number of metrics in the agument.  
+ * the number of metrics in the argument.  
  */
 PCP_CALL extern int pmFetch(int, pmID *, pmResult **);
-PCP_CALL extern int pmFetchHighRes(int, pmID *, pmHighResResult **);
-/* older name maintained for backwards compatibility */
-PCP_CALL extern int pmHighResFetch(int, pmID *, pmHighResResult **);
 
 /*
  * PMCD state changes returned as fetch function results for PM_CONTEXT_HOST
@@ -561,7 +583,6 @@ PCP_CALL extern int pmHighResFetch(int, pmID *, pmHighResResult **);
  * Variant that is used to return a result from an archive.
  */
 PCP_CALL extern int pmFetchArchive(pmResult **);
-PCP_CALL extern int pmFetchHighResArchive(pmHighResResult **);
 
 /*
  * Support for metric values annotated with name:value pairs (labels).
@@ -656,6 +677,7 @@ typedef struct pmTimespec {
 #define PM_LOG_VERS03		0x3
 #define PM_LOG_VOL_TI		-2	/* temporal index */
 #define PM_LOG_VOL_META		-1	/* meta data */
+#define PM_LOG_VOL_CURRENT	0	/* data, current active volume */
 #define PM_LOG_MAXHOSTLEN	64	/* V2 only, deprecated with V3 */
 #define PM_TZ_MAXLEN		40	/* V2 only, deprecated with V3 */
 #define PM_MAX_HOSTNAMELEN	256	/* max supported for V3 onward */
@@ -671,33 +693,22 @@ typedef struct pmTimespec {
 #define PM_LOG_FEATURES		(PM_LOG_FEATURE_NONE | PM_LOG_FEATURE_QA)
 
 typedef struct pmLogLabel {
-    int		ll_magic;	/* PM_LOG_MAGIC | archive format version no. */
-    pid_t	ll_pid;				/* PID of logger */
-    struct timeval	ll_start;		/* start of this archive */
-    char	ll_hostname[PM_LOG_MAXHOSTLEN];	/* name of collection host */
-    char	ll_tz[PM_TZ_MAXLEN];		/* $TZ at collection host */
-} pmLogLabel;
-
-typedef struct pmHighResLogLabel {
     int		magic;	/* PM_LOG_MAGIC | archive format version no. */
     pid_t	pid;		/* PID of logger */
     struct timespec start;	/* start of this archive */
     char	hostname[PM_MAX_HOSTNAMELEN];	/* collection host full name */
     char	timezone[PM_MAX_TIMEZONELEN];	/* generic, squashed $TZ */
     char	zoneinfo[PM_MAX_ZONEINFOLEN];	/* local platform $TZ */
-} pmHighResLogLabel;
+} pmLogLabel;
 
 /*
  * Get the label record from the current archive context, and discover
  * when the archive ends
  */
-PCP_CALL extern int pmGetHighResArchiveLabel(pmHighResLogLabel *);
-PCP_CALL extern int pmGetHighResArchiveEnd(struct timespec *);
 PCP_CALL extern int pmGetArchiveLabel(pmLogLabel *);
-PCP_CALL extern int pmGetArchiveEnd(struct timeval *);
+PCP_CALL extern int pmGetArchiveEnd(struct timespec *);
 
 /* Free result buffer */
-PCP_CALL extern void pmFreeHighResResult(pmHighResResult *);
 PCP_CALL extern void pmFreeResult(pmResult *);
 
 /* Value extract from pmValue and type conversion */
@@ -712,23 +723,16 @@ PCP_CALL extern int pmConvScale(int, const pmAtomValue *, const pmUnits *, pmAto
 
 /* Sort instances for each metric within a pmResult */
 PCP_CALL extern void pmSortInstances(pmResult *);
-PCP_CALL extern void pmSortHighResInstances(pmHighResResult *);
 
 /* Adjust collection time and/or mode for pmFetch */
-PCP_CALL extern int pmSetMode(int, const struct timeval *, int);
-PCP_CALL extern int pmSetModeHighRes(int, const struct timespec *, const struct timespec *);
+PCP_CALL extern int pmSetMode(int, const struct timespec *, const struct timespec *);
 #define PM_MODE_LIVE	0
 #define PM_MODE_INTERP	1
 #define PM_MODE_FORW	2
 #define PM_MODE_BACK	3
-/* Extended time base definitions and macros - for pmSetMode(3) only */
-#define PM_XTB_FLAG	0x1000000
-#define PM_XTB_SET(m)	(PM_XTB_FLAG | ((m) << 16))
-#define PM_XTB_GET(m)	(((m) & PM_XTB_FLAG) ? (((m) & 0xff0000) >> 16) : -1)
 
 /* Modify the value of one or more metrics */
 PCP_CALL extern int pmStore(const pmResult *);
-PCP_CALL extern int pmStoreHighRes(const pmHighResResult *);
 
 /* Get help and descriptive text */
 PCP_CALL extern int pmLookupText(pmID, int, char **);
@@ -769,13 +773,8 @@ PCP_CALL extern const char *pmEventFlagsStr(int);		/* NOT thread-safe */
 PCP_CALL extern char *pmEventFlagsStr_r(int, char *, int);
 
 /* Parse -t, -S, -T, -A and -O options */
-PCP_CALL extern int pmParseInterval(const char *, struct timeval *, char **);
-PCP_CALL extern int pmParseHighResInterval(const char *, struct timespec *, char **);
+PCP_CALL extern int pmParseInterval(const char *, struct timespec *, char **);
 PCP_CALL extern int pmParseTimeWindow(
-      const char *, const char *, const char *, const char *,
-      const struct timeval *, const struct timeval *,
-      struct timeval *, struct timeval *, struct timeval *, char **);
-PCP_CALL extern int pmParseHighResTimeWindow(
       const char *, const char *, const char *, const char *,
       const struct timespec *, const struct timespec *,
       struct timespec *, struct timespec *, struct timespec *, char **);
@@ -887,7 +886,7 @@ PCP_CALL extern int pmGetVersion(void);
 			"metrics source is a PCP archive" }
 #define PMLONGOPT_DEBUG		"debug"
 #define PMOPT_DEBUG	{ PMLONGOPT_DEBUG,	1, 'D',	"DBG", \
-			NULL }
+			"set debug options, see pmdbg(1)" }
 #define PMLONGOPT_GUIMODE	"guimode"
 #define PMOPT_GUIMODE	{ PMLONGOPT_GUIMODE,	0, 'g',	0, \
 			"start in GUI mode with new time control" }
@@ -1039,14 +1038,10 @@ typedef struct pmOptions {
     int			narchives;
     char **		hosts;
     char **		archives;
-#if PMAPI_VERSION == PMAPI_VERSION_3
-    struct timeval	unused[4];
-#else
-    struct timeval	start;
-    struct timeval	finish;
-    struct timeval	origin;
-    struct timeval	interval;
-#endif
+    struct timespec	start;
+    struct timespec	finish;
+    struct timespec	origin;
+    struct timespec	interval;
     char *		align_optarg;
     char *		start_optarg;
     char *		finish_optarg;
@@ -1061,12 +1056,6 @@ typedef struct pmOptions {
     unsigned int	nsflag  : 1;
     unsigned int	Lflag   : 1;
     unsigned int	zeroes  : 28;
-#if PMAPI_VERSION == PMAPI_VERSION_3
-    struct timespec	start;
-    struct timespec	finish;
-    struct timespec	origin;
-    struct timespec	interval;
-#endif
 } pmOptions;
 
 PCP_CALL extern int pmgetopt_r(int, char *const *, pmOptions *);
@@ -1164,16 +1153,16 @@ typedef struct pmHighResEventArray {
 } pmHighResEventArray;
 
 /* Unpack a PM_TYPE_EVENT value into a set on pmResults */
-PCP_CALL extern int pmUnpackEventRecords(pmValueSet *, int, pmResult ***);
+PCP_CALL extern int pmUnpackEventRecords(pmValueSet *, int, pmResult_v2 ***);
 
 /* Free set of pmResults from pmUnpackEventRecords */
-PCP_CALL extern void pmFreeEventResult(pmResult **);
+PCP_CALL extern void pmFreeEventResult(pmResult_v2 **);
 
-/* Unpack a PM_TYPE_HIGHRES_EVENT value into a set on pmHighResResults */
-PCP_CALL extern int pmUnpackHighResEventRecords(pmValueSet *, int, pmHighResResult ***);
+/* Unpack a PM_TYPE_HIGHRES_EVENT value into a set on pmResults */
+PCP_CALL extern int pmUnpackHighResEventRecords(pmValueSet *, int, pmResult ***);
 
-/* Free set of pmHighResResults from pmUnpackEventRecords */
-PCP_CALL extern void pmFreeHighResEventResult(pmHighResResult **);
+/* Free set of pmResults from pmUnpackEventRecords */
+PCP_CALL extern void pmFreeHighResEventResult(pmResult **);
 
 /* Service discovery, for clients. */
 #define PM_SERVER_SERVICE_SPEC	"pmcd"
@@ -1196,16 +1185,19 @@ PCP_CALL extern int pmExtendFetchGroup_event(pmFG, const char *, const char *,
 			const char *, const char *,
 			struct timespec[], pmAtomValue[], int, int[],
 			unsigned int, unsigned int *, int *);
-PCP_CALL extern int pmExtendFetchGroup_timestamp(pmFG, struct timeval *);
+PCP_CALL extern int pmExtendFetchGroup_timeval(pmFG, struct timeval *);
+PCP_CALL extern int pmExtendFetchGroup_timespec(pmFG, struct timespec *);
 PCP_CALL extern int pmFetchGroup(pmFG);
 PCP_CALL extern int pmDestroyFetchGroup(pmFG);
 
 /* libpcp debug/tracing */
 PCP_CALL extern int pmSetDebug(const char *);
 PCP_CALL extern int pmClearDebug(const char *);
+PCP_CALL extern int pmDebug(const char *);
+PCP_CALL extern char *pmGetDebug(void);
 
 /*
- * New style ...
+ * Debug options.
  * Note that comments are important ... these are extracted and
  * built into pmdbg.h.
  * For the "add a new debug flag" recipe, see ../../libpcp/src/mk.pmdbg
@@ -1240,7 +1232,6 @@ typedef struct {
     int	attr;		/* Connection attribute handling */
     int	http;		/* Trace HTTP operations in libpcp_web */
     int	desperate;	/* Verbose/Desperate level (developers only) */
-/* new ones start here, no DBG_TRACE_xxx macro and no backwards compatibility */
     int	deprecated;	/* Report use of deprecated services */
     int	exec;	 	/* __pmProcessExec and related calls */
     int labels;		/* Metric label metadata operations */
@@ -1267,6 +1258,7 @@ typedef struct {
     int	tls;		/* Transport Layer Security operations */
     int	misc;		/* Miscellaneous odds and sods */
     int	qed;		/* Methods in libpcp_qed */
+    int getopt;		/* Processing of command-line arguments */
 } pmdebugoptions_t;
 
 PCP_DATA extern pmdebugoptions_t	pmDebugOptions;
@@ -1338,7 +1330,7 @@ PCP_CALL extern double pmtimevalAdd(const struct timeval *, const struct timeval
 PCP_CALL extern double pmtimevalSub(const struct timeval *, const struct timeval *);
 PCP_CALL extern double pmtimevalToReal(const struct timeval *);
 PCP_CALL extern void pmtimevalFromReal(double, struct timeval *);
-PCP_CALL extern void pmPrintStamp(FILE *, const struct timeval *);
+PCP_CALL extern void pmtimevalPrint(FILE *, const struct timeval *);
 
 /* struct timespec manipulations */
 PCP_CALL extern int pmtimespecNow(struct timespec *);
@@ -1348,7 +1340,12 @@ PCP_CALL extern double pmtimespecAdd(const struct timespec *, const struct times
 PCP_CALL extern double pmtimespecSub(const struct timespec *, const struct timespec *);
 PCP_CALL extern double pmtimespecToReal(const struct timespec *);
 PCP_CALL extern void pmtimespecFromReal(double, struct timespec *);
-PCP_CALL extern void pmPrintHighResStamp(FILE *, const struct timespec *);
+PCP_CALL extern void pmtimespecPrint(FILE *, const struct timespec *);
+PCP_CALL extern void pmtimespecPrintInterval(FILE *, const struct timespec *);
+
+/* timespec <-> timeval conversions */
+PCP_CALL extern void pmtimevalTotimespec(struct timeval *, struct timespec *);
+PCP_CALL extern void pmtimespecTotimeval(struct timespec *, struct timeval *);
 
 /* filesystem path name separator */
 PCP_CALL extern int pmPathSeparator(void);
@@ -1365,6 +1362,160 @@ PCP_CALL extern int pmGetUsername(char **);
 /* DSO PMDA helpers */
 PCP_CALL extern char *pmSpecLocalPMDA(const char *);
 
+/*
+ * PMAPI_VERSION_2 interfaces
+ */
+PCP_CALL extern int pmGetArchiveEnd_v2(struct timeval *);
+
+struct pmOptions_v2;
+typedef int (*pmOptionOverride_v2)(int, struct pmOptions_v2 *);
+
+typedef struct pmOptions_v2 {
+    int			version;
+    int			flags;
+
+    /* in: define set of all options */
+    const char *	short_options;
+    pmLongOptions *	long_options;
+    const char *	short_usage;
+
+    /* in: method for general override */
+    pmOptionOverride_v2	override;
+
+    /* out: usual getopt information */
+    int			index;
+    int			optind;
+    int			opterr;
+    int			optopt;
+    char		*optarg;
+
+    /* internals; do not ever access */
+    int			__initialized;
+    char *		__nextchar;
+    int			__ordering;
+    int			__posixly_correct;
+    int			__first_nonopt;
+    int			__last_nonopt;
+
+    /* out: error count */
+    int 		errors;
+
+    /* out: PMAPI options and values */
+    int			context;	/* PM_CONTEXT_{HOST,ARCHIVE,LOCAL} */
+    int			nhosts;
+    int			narchives;
+    char **		hosts;
+    char **		archives;
+    struct timeval	start;
+    struct timeval	finish;
+    struct timeval	origin;
+    struct timeval	interval;
+    char *		align_optarg;
+    char *		start_optarg;
+    char *		finish_optarg;
+    char *		origin_optarg;
+    char *		guiport_optarg;
+    char *		timezone;
+    int			samples;
+    int			guiport;
+    int			padding;
+    unsigned int	guiflag : 1;
+    unsigned int	tzflag  : 1;
+    unsigned int	nsflag  : 1;
+    unsigned int	Lflag   : 1;
+    unsigned int	zeroes  : 28;
+} pmOptions_v2;
+
+PCP_CALL extern int pmgetopt_r_v2(int, char *const *, pmOptions_v2 *);
+PCP_CALL extern int pmGetOptions_v2(int, char *const *, pmOptions_v2 *);
+PCP_CALL extern int pmGetContextOptions_v2(int, pmOptions_v2 *);
+PCP_CALL extern void pmUsageMessage_v2(pmOptions_v2 *);
+PCP_CALL extern void pmFreeOptions_v2(pmOptions_v2 *);
+
+PCP_CALL extern int pmParseTimeWindow_v2(
+      const char *, const char *, const char *, const char *,
+      const struct timeval *, const struct timeval *,
+      struct timeval *, struct timeval *, struct timeval *, char **);
+
+
+typedef struct pmLogLabel_v2 {
+    int		ll_magic;	/* PM_LOG_MAGIC | archive format version no. */
+    pid_t	ll_pid;				/* PID of logger */
+    struct timeval	ll_start;		/* start of this archive */
+    char	ll_hostname[PM_LOG_MAXHOSTLEN];	/* name of collection host */
+    char	ll_tz[PM_TZ_MAXLEN];		/* $TZ at collection host */
+} pmLogLabel_v2;
+
+PCP_CALL extern int pmGetArchiveLabel_v2(pmLogLabel_v2 *);
+PCP_CALL extern int pmParseInterval_v2(const char *, struct timeval *, char **);
+PCP_CALL extern int pmSetMode_v2(int, const struct timeval *, int);
+
+PCP_CALL extern int pmFetch_v2(int, pmID *, pmResult_v2 **);
+PCP_CALL extern void pmFreeResult_v2(pmResult_v2 *);
+PCP_CALL extern int pmFetchArchive_v2(pmResult_v2 **);
+PCP_CALL extern int pmStore_v2(const pmResult_v2 *);
+PCP_CALL extern void pmSortInstances_v2(pmResult_v2 *);
+
+#if PMAPI_VERSION == PMAPI_VERSION_2
+/*
+ * old names with API changes mapped to _v2 variants
+ */
+#define pmGetArchiveEnd pmGetArchiveEnd_v2
+#define pmOptionOverride pmOptionOverride_v2
+#define pmOptions pmOptions_v2
+#define pmgetopt_r pmgetopt_r_v2
+#define pmGetOptions pmGetOptions_v2
+#define pmGetContextOptions pmGetContextOptions_v2
+#define pmUsageMessage pmUsageMessage_v2
+#define pmFreeOptions pmFreeOptions_v2
+#define pmParseTimeWindow pmParseTimeWindow_v2
+#define pmLogLabel pmLogLabel_v2
+#define pmGetArchiveLabel pmGetArchiveLabel_v2
+#define pmParseInterval pmParseInterval_v2
+#define pmSetMode pmSetMode_v2
+#define pmResult pmResult_v2
+#define pmHighResResult pmResult
+#define pmFetch pmFetch_v2
+#define pmFetchHighRes pmFetch
+#define pmFreeResult pmFreeResult_v2
+#define pmFreeHighResResult pmFreeResult
+#define pmFetchArchive pmFetchArchive_v2
+#define pmStore pmStore_v2
+#define pmSortInstances pmSortInstances_v2
+#define pmPrintStamp pmtimevalPrint
+#define pmPrintHighResStamp pmtimespecPrint
+#define pmExtendFetchGroup_timestamp pmExtendFetchGroup_timeval
+
+/*
+ * Extended time base definitions and macros
+ * - only for deprecated pmSetMode_v2()
+ */
+#define PM_XTB_FLAG	0x1000000
+#define PM_XTB_SET(m)	(PM_XTB_FLAG | ((m) << 16))
+#define PM_XTB_GET(m)	(((m) & PM_XTB_FLAG) ? (((m) & 0xff0000) >> 16) : -1)
+#endif
+
+/* bad name, preserved for Version 2 */
+#define pmHighResFetch pmFetch
+
+#if PMAPI_VERSION >= PMAPI_VERSION_4
+/*
+ * alias mappings for HighRes and renamed interfaces
+ */
+#define pmGetHighResArchiveEnd pmGetArchiveEnd
+#define pmParseHighResTimeWindow pmParseTimeWindow
+#define pmGetHighResArchiveLabel pmGetArchiveLabel
+#define pmParseHighResInterval pmParseInterval
+#define pmSetModeHighRes pmSetMode
+#define pmHighResResult pmResult
+#define pmFetchHighRes pmFetch
+#define pmFetchHighResArchive pmFetchArchive
+#define pmStoreHighRes pmStore
+#define pmSortHighResInstances pmSortInstances
+#define pmPrintStamp pmtimevalPrint
+#define pmPrintHighResStamp pmtimespecPrint
+#endif
+
 #ifdef __cplusplus
 }
 #endif
