@@ -60,6 +60,8 @@ extern int __pmGetDate(struct timespec *, char const *, struct timespec const *)
 #define __ntohpmLabel(a)	/* noop */
 #define __htonpmValueBlock(a)	/* noop */
 #define __ntohpmValueBlock(a)	/* noop */
+#define __ntohpmValueBlock_hdr(a)	/* noop */
+#define __ntohpmValueBlock_buf(a)	/* noop */
 #define __htonf(a)		/* noop */
 #define __ntohf(a)		/* noop */
 #define __htond(a)		/* noop */
@@ -90,6 +92,8 @@ extern void __htonpmLabel(pmLabel * const) _PCP_HIDDEN;
 extern void __ntohpmLabel(pmLabel * const) _PCP_HIDDEN;
 extern void __htonpmValueBlock(pmValueBlock * const) _PCP_HIDDEN;
 extern void __ntohpmValueBlock(pmValueBlock * const) _PCP_HIDDEN;
+extern void __ntohpmValueBlock_hdr(pmValueBlock * const) _PCP_HIDDEN;
+extern void __ntohpmValueBlock_buf(pmValueBlock * const) _PCP_HIDDEN;
 extern void __htonf(char *) _PCP_HIDDEN;	/* float */
 #define __ntohf(v) __htonf(v)
 #define __htond(v) __htonll(v)			/* double */
@@ -336,7 +340,6 @@ extern int pmStore_ctx(__pmContext *, const __pmResult *) _PCP_HIDDEN;
 extern int __pmSendResult_ctx(__pmContext *, int, int, const __pmResult *) _PCP_HIDDEN;
 extern int __pmSendHighResResult_ctx(__pmContext *, int, int, const __pmResult *) _PCP_HIDDEN;
 extern void __pmDumpResult_ctx(__pmContext *, FILE *, const pmResult *) _PCP_HIDDEN;
-extern void __pmDumpHighResResult_ctx(__pmContext *, FILE *, const pmHighResResult *) _PCP_HIDDEN;
 extern void __pmPrintResult_ctx(__pmContext *, FILE *, const __pmResult *) _PCP_HIDDEN;
 extern int pmGetArchiveEnd_ctx(__pmContext *, __pmTimestamp *) _PCP_HIDDEN;
 extern int __pmGetArchiveEnd_ctx(__pmContext *, __pmTimestamp *) _PCP_HIDDEN;
@@ -375,8 +378,6 @@ extern int __pmIsLocked(void *) _PCP_HIDDEN;
 extern void __pmCheckIsUnlocked(void *, char *, int) _PCP_HIDDEN;
 #endif /* BUILD_WITH_LOCK_ASSERTS */
 
-extern int __pmGetPDUCeiling(void) _PCP_HIDDEN;
-
 extern int __pmSetFeaturesIPC(int, int, int) _PCP_HIDDEN;
 extern int __pmSetDataIPC(int, void *) _PCP_HIDDEN;
 extern int __pmDataIPCSize(void) _PCP_HIDDEN;
@@ -453,4 +454,19 @@ extern void __pmParseTimeWindow3(pmOptions *,
 extern void __pmSetSampleInterval2(pmOptions *, char *) _PCP_HIDDEN;
 extern void __pmSetSampleInterval3(pmOptions *, char *) _PCP_HIDDEN;
 
+/* hook for win32.c initialization */
+extern void __pmSetProgname(const char *) _PCP_HIDDEN;
+
+/*
+ * PMAPI_VERSION_2 interfaces
+ */
+extern void __pmDumpResult_ctx_v2(__pmContext *, FILE *, const pmResult_v2 *) _PCP_HIDDEN;
+
+#if PMAPI_VERSION >= PMAPI_VERSION_4
+/*
+ * retire HighRes interfaces
+ */
+#define __pmDumpHighResResult_ctx __pmDumpResult_ctx
+#endif
+
 #endif /* _LIBPCP_INTERNAL_H */
