@@ -223,7 +223,7 @@ shellProbeForServices(const char *script, const char *service,
     int			sts;
     pthread_t		*threads = NULL;
     unsigned		threadIx;
-    unsigned		nThreads;
+    unsigned		nThreads = 0;
     pthread_attr_t	threadAttr;
 #endif
 
@@ -468,8 +468,7 @@ parseFile(const char *option, char *buffer, int buflen)
 {
     char		*end;
 
-    strncpy(buffer, option, buflen);
-    buffer[buflen-1] = '\0';
+    pmstrncpy(buffer, buflen, option);
 
     /* Remove back-slash escaped comma characters, if any. */
     for (end = buffer; *end != '\0' && *end != ','; end++) {
