@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2013-2015 Red Hat.
+ * Copyright (c) 2013-2015,2021 Red Hat.
  * Copyright (c) 2007 Aconex.  All Rights Reserved.
  * Copyright (c) 1995-2002 Silicon Graphics, Inc.  All Rights Reserved.
  * 
@@ -338,7 +338,7 @@ static char *
 hostStrndup(const char *name, int namelen)
 {
     char *s = malloc(namelen + 1);
-    strncpy(s, name, namelen);
+    memcpy(s, name, namelen);
     s[namelen] = '\0';
     return s;
 }
@@ -537,8 +537,11 @@ parseSocketPath(
     else {
 	path = start;
 	len = s - start;
-	if (len >= MAXPATHLEN)
+	if (len >= MAXPATHLEN) {
 	    len = MAXPATHLEN - 1;
+	    if (pmDebugOptions.context)
+		fprintf(stderr, "parseSocketPath(spec=%s, ...): path truncated [case 1] to %" FMT_SIZE " bytes\n", spec, len);
+	}
     }
 
     /*
@@ -546,11 +549,14 @@ parseSocketPath(
      * (optional) "//" from "local://some/path".
      */
     if (*path != pmPathSeparator()) {
+	if (len > MAXPATHLEN - 2) {
+	    len = MAXPATHLEN - 2;
+	    if (pmDebugOptions.context)
+		fprintf(stderr, "parseSocketPath(spec=%s, ...): path truncated [case 2] to %" FMT_SIZE " bytes\n", spec, len);
+	}
 	absolute_path[0] = pmPathSeparator();
-	strncpy(absolute_path + 1, path, len);
-	if (len < sizeof(absolute_path)-1)
-	    absolute_path[++len] = '\0';
-	absolute_path[sizeof(absolute_path)-1] = '\0';
+	memcpy(absolute_path+1, path, len);
+	absolute_path[++len] = '\0';
 	path = absolute_path;
     }
 
@@ -855,15 +861,25 @@ parseAttributeSpec(
 	    if ((*s == '\0' || *s == '/') && s == start)
 		break;
 	    len = v ? (v - start - 1) : (s - start);
-	    buflen = (len < sizeof(buffer)-1) ? len : sizeof(buffer)-1;
-	    strncpy(buffer, start, buflen);
+	    if (len > sizeof(buffer) - 1) {
+		buflen = sizeof(buffer) - 1;
+		if (pmDebugOptions.context) {
+		    fprintf(stderr, "parseAttributeSpec(spec=%s, ...): attribute \"%*.*s\" too long, truncated to \"%*.*s\"\n",
+			spec, len, len, start,
+			buflen, buflen, start);
+		}
+	    }
+	    else {
+		buflen = len;
+	    }
+	    memcpy(buffer, start, buflen);
 	    buffer[buflen] = '\0';
 	    attr = __pmLookupAttrKey(buffer, buflen+1);
 	    if (attr != PCP_ATTR_NONE) {
 		char *val = NULL;
+		int val_len = s - v;
 
-		if (v && (val = strndup(v, s - v)) == NULL) {
-		    sts = -ENOMEM;
+		if (v && (sts = __pmUrlDecode(v, val_len, &val)) < 0) {
 		    goto fail;
 		}
 		if ((sts = __pmHashAdd(attr, (void *)val, attributes)) < 0) {
@@ -871,6 +887,12 @@ parseAttributeSpec(
 		    goto fail;
 		}
 	    }
+	    else {
+		if (pmDebugOptions.context) {
+		    fprintf(stderr, "parseAttributeSpec(spec=%s, ...): \"%s\" is not a valid attribute name\n",
+			spec, buffer);
+		}
+	    }
 	    v = NULL;
 	    if (*s == '\0' || *s == '/')
 		break;
@@ -1010,6 +1032,8 @@ __pmAttrStr_r(__pmAttrKey key, const char *data, char *string, size_t size)
 {
     char name[16];	/* must be sufficient to hold any key name (above) */
     int sts;
+    char *encoded;
+    int attr_str_len;
 
     if ((sts = __pmAttrKeyStr_r(key, name, sizeof(name))) <= 0)
 	return sts;
@@ -1025,7 +1049,12 @@ __pmAttrStr_r(__pmAttrKey key, const char *data, char *string, size_t size)
     case PCP_ATTR_GROUPID:
     case PCP_ATTR_PROCESSID:
     case PCP_ATTR_CONTAINER:
-	return pmsprintf(string, size, "%s=%s", name, data ? data : "");
+	if ((sts = __pmUrlEncode(data ? data : "", data ? strlen(data) : 0, &encoded)) < 0)
+	    return 0;
+	attr_str_len = pmsprintf(string, size, "%s=%s", name, encoded);
+	free(encoded);
+	return attr_str_len;
+
 
     case PCP_ATTR_UNIXSOCK:
     case PCP_ATTR_LOCAL:
