@@ -199,7 +199,7 @@ probeForServices(const char *service,
     int			sts;
     pthread_t		*threads = NULL;
     unsigned		threadIx;
-    unsigned		nThreads;
+    unsigned		nThreads = 0;
     pthread_attr_t	threadAttr;
 #endif
 
