@@ -23,7 +23,7 @@
  *	attempts to set/clear the state in pmSyslog() which locking will
  *	not avoid
  *
- * pmProgname - most likely set in main(), not worth protecting here
+ * myprogname - most likely set in main(), not worth protecting here
  *
  * base (in __pmProcessDataSize) - no real side-effects, don't bother
  *	locking
@@ -60,7 +60,6 @@
 #include "deprecated.h"
 #include "pmdbg.h"
 #include "internal.h"
-#include "deprecated.h"
 
 #if defined(HAVE_SYS_TIMES_H)
 #include <sys/times.h>
@@ -90,11 +89,9 @@ static int	pmState = PM_STATE_APPL;
 static int	done_exit;
 static int	xconfirm_init;
 static char 	*xconfirm;
+static char	*myprogname = "pcp";		/* the real McCoy */
 
-PCP_DATA char	*pmProgname = "pcp";		/* the real McCoy */
-
-PCP_DATA int	pmDebug;			/* the real McCoy ... old style */
-PCP_DATA pmdebugoptions_t	pmDebugOptions;	/* the real McCoy ... new style */
+PCP_DATA pmdebugoptions_t	pmDebugOptions;	/* the real McCoy */
 
 static int vpmprintf(const char *, va_list);
 
@@ -118,7 +115,7 @@ __pmIsUtilLock(void *lock)
 char *
 pmGetProgname(void)
 {
-    return pmProgname;
+    return myprogname;
 }
 
 /*
@@ -347,8 +344,9 @@ logreopen(const char *progname, const char *logname, FILE *oldstream,
 		char	*p;
 		p = strerror_r(save_error, errmsg, sizeof(errmsg));
 		if (p != errmsg)
-		    strncpy(errmsg, p, sizeof(errmsg));
-		errmsg[sizeof(errmsg)-1] = '\0';
+		    pmstrncpy(errmsg, sizeof(errmsg), p);
+		else
+		    errmsg[sizeof(errmsg)-1] = '\0';
 	    }
 #else
 	    /*
@@ -721,10 +719,7 @@ __pmCtlDebug(int op)
 	for (i = 0; i < num_debug; i++) {
 	    debug_map[i].state = *(debug_map[i].options);
 	    *(debug_map[i].options) = 0;
-	    if (debug_map[i].bit != 0)
-		pmDebug &= ~debug_map[i].bit;
 	}
-	pmDebug = 0;
     }
     else if (op == PM_CTL_DEBUG_RESTORE) {
 	/*
@@ -732,8 +727,6 @@ __pmCtlDebug(int op)
 	 */
 	for (i = 0; i < num_debug; i++) {
 	    *(debug_map[i].options) = debug_map[i].state;
-	    if (debug_map[i].state && debug_map[i].bit != 0)
-		pmDebug |= debug_map[i].bit;
 	}
     }
 }
@@ -812,9 +805,9 @@ dump_valueset(__pmContext *ctxp, FILE *f, pmValueSet *vsp)
     }
 }
 
-/* Internal variant of __pmDumpResult() with current context. */
+/* Internal variant of __pmDumpResult_v2() with current context. */
 void
-__pmDumpResult_ctx(__pmContext *ctxp, FILE *f, const pmResult *resp)
+__pmDumpResult_ctx_v2(__pmContext *ctxp, FILE *f, const pmResult_v2 *resp)
 {
     int		i;
 
@@ -823,7 +816,7 @@ __pmDumpResult_ctx(__pmContext *ctxp, FILE *f, const pmResult *resp)
     __pmCtlDebug(PM_CTL_DEBUG_SAVE);
     fprintf(f, "pmResult dump from " PRINTF_P_PFX "%p timestamp: %d.%06d ",
 	resp, (int)resp->timestamp.tv_sec, (int)resp->timestamp.tv_usec);
-    pmPrintStamp(f, &resp->timestamp);
+    pmtimevalPrint(f, &resp->timestamp);
     fprintf(f, " numpmid: %d\n", resp->numpmid);
     for (i = 0; i < resp->numpmid; i++)
 	dump_valueset(ctxp, f, resp->vset[i]);
@@ -831,9 +824,9 @@ __pmDumpResult_ctx(__pmContext *ctxp, FILE *f, const pmResult *resp)
 }
 
 void
-__pmDumpResult(FILE *f, const pmResult *resp)
+__pmDumpResult_v2(FILE *f, const pmResult_v2 *resp)
 {
-    __pmDumpResult_ctx(NULL, f, resp);
+    __pmDumpResult_ctx_v2(NULL, f, resp);
 }
 
 /* Internal variant of __pmPrintResult() with current context. */
@@ -861,9 +854,9 @@ __pmPrintResult(FILE *f, const __pmResult *resp)
     __pmPrintResult_ctx(NULL, f, resp);
 }
 
-/* Internal variant of __pmDumpHighResResult() with current context. */
+/* Internal variant of __pmDumpResult() with current context. */
 void
-__pmDumpHighResResult_ctx(__pmContext *ctxp, FILE *f, const pmHighResResult *hresp)
+__pmDumpResult_ctx(__pmContext *ctxp, FILE *f, const pmResult *hresp)
 {
     int		i;
 
@@ -871,11 +864,11 @@ __pmDumpHighResResult_ctx(__pmContext *ctxp, FILE *f, const pmHighResResult *hre
 	PM_ASSERT_IS_LOCKED(ctxp->c_lock);
 
     __pmCtlDebug(PM_CTL_DEBUG_SAVE);
-    fprintf(f, "%s dump from " PRINTF_P_PFX "%p timestamp: %lld.%09lld ",
-	    "pmHighResResult", hresp,
+    fprintf(f, "pmResult dump from " PRINTF_P_PFX "%p timestamp: %lld.%09lld ",
+	    hresp,
 	    (long long)hresp->timestamp.tv_sec,
 	    (long long)hresp->timestamp.tv_nsec);
-    pmPrintHighResStamp(f, &hresp->timestamp);
+    pmtimespecPrint(f, &hresp->timestamp);
     fprintf(f, " numpmid: %d\n", hresp->numpmid);
     for (i = 0; i < hresp->numpmid; i++)
 	dump_valueset(ctxp, f, hresp->vset[i]);
@@ -883,9 +876,9 @@ __pmDumpHighResResult_ctx(__pmContext *ctxp, FILE *f, const pmHighResResult *hre
 }
 
 void
-__pmDumpHighResResult(FILE *f, const pmHighResResult *hresp)
+__pmDumpResult(FILE *f, const pmResult *hresp)
 {
-    __pmDumpHighResResult_ctx(NULL, f, hresp);
+    __pmDumpResult_ctx(NULL, f, hresp);
 }
 
 static void
@@ -961,9 +954,9 @@ print_event_summary(FILE *f, const pmValue *val, int highres)
 	fputc(' ', f);
 
 	if (highres)
-	    pmPrintHighResStamp(f, &tsstamp);
+	    pmtimespecPrint(f, &tsstamp);
 	else
-	    pmPrintStamp(f, &tvstamp);
+	    pmtimevalPrint(f, &tvstamp);
 
 	if (nrecords > 1) {
 	    fprintf(f, "...");
@@ -971,13 +964,13 @@ print_event_summary(FILE *f, const pmValue *val, int highres)
 		tsp = (pmTimespec *)base;
 		tsstamp.tv_sec = tsp->tv_sec;
 		tsstamp.tv_nsec = tsp->tv_nsec;
-		pmPrintHighResStamp(f, &tsstamp);
+		pmtimespecPrint(f, &tsstamp);
 	    }
 	    else {
 		tvp = (pmTimeval *)base;
 		tvstamp.tv_sec = tvp->tv_sec;
 		tvstamp.tv_usec = tvp->tv_usec;
-		pmPrintStamp(f, &tvstamp);
+		pmtimevalPrint(f, &tvstamp);
 	    }
 	}
     }
@@ -1375,7 +1368,7 @@ __pmTimestampDec(__pmTimestamp *ap, const __pmTimestamp *bp)
  *       to usec precision.
  */
 void
-pmPrintStamp(FILE *f, const struct timeval *tp)
+pmtimevalPrint(FILE *f, const struct timeval *tp)
 {
     struct tm	tmp;
     time_t	now;
@@ -1389,7 +1382,7 @@ pmPrintStamp(FILE *f, const struct timeval *tp)
  * print high resolution timestamp in HH:MM:SS.XXXXXXXXX format
  */
 void
-pmPrintHighResStamp(FILE *f, const struct timespec *tp)
+pmtimespecPrint(FILE *f, const struct timespec *tp)
 {
     struct tm	tmp;
     time_t	now;
@@ -1443,6 +1436,48 @@ __pmPrintTimestamp(FILE *f, const __pmTimestamp *tsp)
     fprintf(f, "%02d:%02d:%02d.%09d", tmp.tm_hour, tmp.tm_min, tmp.tm_sec, (int)tsp->nsec);
 }
 
+/*
+ * print interval timespec in [[Nh]Nm]n[.N]s format
+ */
+void
+pmtimespecPrintInterval(FILE *f, const struct timespec *tsp)
+{
+    struct timespec	interval;
+
+    if (tsp == NULL) {
+	fprintf(stderr, "NULL");
+	return;
+    }
+    interval = *tsp;
+    if (interval.tv_sec < 0) {
+	fprintf(stderr, "-");
+	interval.tv_sec = -interval.tv_sec;
+    }
+    if (interval.tv_nsec < 0) {
+	fprintf(stderr, "-");
+	interval.tv_nsec = -interval.tv_nsec;
+    }
+    if (interval.tv_sec > 3600) {
+	fprintf(f, "%dh", (int)(interval.tv_sec / 3600));
+	interval.tv_sec = interval.tv_sec % 3600;
+    }
+    if (interval.tv_sec > 60) {
+	fprintf(f, "%dm", (int)(interval.tv_sec / 60));
+	interval.tv_sec = interval.tv_sec % 60;
+    }
+    fprintf(f, "%d", (int)interval.tv_sec);
+    if (interval.tv_nsec != 0) {
+	long	val = interval.tv_nsec;
+	int	prec = 9;
+	while (prec > 0 && (val % 10) == 0) {
+	    prec--;
+	    val /= 10;
+	}
+	fprintf(f, ".%0*ld", prec, val);
+    }
+    fprintf(f, "s");
+}
+
 /*
  * must be in agreement with ordinal values for PM_TYPE_* #defines
  */
@@ -1577,71 +1612,47 @@ pmPrintDesc(FILE *f, const pmDesc *desc)
 
 #define DEBUG_CLEAR 0
 #define DEBUG_SET 1
-#define DEBUG_OLD 0
-#define DEBUG_NEW 1
+#define DEBUG_TEST 2
 
 static int
-debug(const char *spec, int action, int style)
+debug(const char *spec, int action)
 {
-    int		val = 0;
-    int		tmp;
     const char	*p;
     char	*pend;
     int		i;
-    int		sts = 0;	/* for DEBUG_NEW interface */
+    int		sts = 0;
 
     /* save old state, so calls are additive */
     for (i = 0; i < num_debug; i++)
 	debug_map[i].state = *(debug_map[i].options);
 
     for (p = spec; *p; ) {
-	/*
-	 * backwards compatibility, "string" may be a number for setting
-	 * bit fields directly
-	 */
-	tmp = (int)strtol(p, &pend, 10);
-	if (pend > p) {
-	    /* found a number */
-	    if (tmp == -1)
-		/* special case ... -1 really means set all the bits! */
-		val = INT_MAX;
-	    else
-		val |= tmp;
-	    /* for all matching bits, set/clear the corresponding new field */
-	    for (i = 0; i < num_debug; i++) {
-		if (val & debug_map[i].bit)
-		    debug_map[i].state = (action == DEBUG_SET ? 1 : 0);
-	    }
-	    if (*pend == ',') {
-		p = pend + 1;
-		continue;
-	    }
-	    else if (*pend == '\0')
-		break;
-	    /* something bogus after a number ... */
-	    sts = PM_ERR_CONV;
-	    break;
-	}
-
 	pend = strchr(p, ',');
 	if (pend == NULL)
 	    pend = (char *)&p[strlen(p)];
 
 	if (pend-p == 3 && strncasecmp(p, "all", pend-p) == 0) {
-	    val |= INT_MAX;
-	    for (i = 0; i < num_debug; i++) {
-		debug_map[i].state = (action == DEBUG_SET ? 1 : 0);
+	    if (action == DEBUG_TEST) {
+		for (i = 0; i < num_debug; i++) {
+		    if (*(debug_map[i].options) == 0)
+			return 0;
+		}
+		return 1;
+	    }
+	    else {
+		for (i = 0; i < num_debug; i++) {
+		    debug_map[i].state = (action == DEBUG_SET ? 1 : 0);
+		}
 	    }
 	}
 	else {
 	    for (i = 0; i < num_debug; i++) {
 		if (pend-p == strlen(debug_map[i].name) &&
 		    strncasecmp(p, debug_map[i].name, pend-p) == 0) {
-		    if (debug_map[i].bit != 0)
-			/* has corresponding old-stype bit field */
-			val |= debug_map[i].bit;
-		    debug_map[i].state = (action == DEBUG_SET ? 1 : 0);
-		    break;
+			if (action == DEBUG_TEST && (*(debug_map[i].options) == 1))
+			    return 1;
+			debug_map[i].state = (action == DEBUG_SET ? 1 : 0);
+			break;
 		}
 	    }
 	    if (i == num_debug) {
@@ -1656,19 +1667,14 @@ debug(const char *spec, int action, int style)
     }
 
     if (sts == 0) {
-	/* all's well, now set the options and bits */
+	if (action == DEBUG_TEST)
+	    /* none of the given debug options is enabled */
+	    return 0;
+	/* all's well, now set the options */
 	for (i = 0; i < num_debug; i++)
 	    *(debug_map[i].options) = debug_map[i].state;
-	/* set/clear old-style bit mask */
-	if (action == DEBUG_SET)
-	    pmDebug |= val;
-	else
-	    pmDebug &= ~val;
     }
 
-    if (style == DEBUG_OLD && sts == 0)
-	return val;
-
     return sts;
 }
 
@@ -1678,23 +1684,6 @@ __pmDumpDebug(FILE *f)
     int		i;
     int		nset;
 
-    nset = 0;
-    fprintf(f, "pmDebug:\t");
-    if (pmDebug == 0)
-	fprintf(f, "Nothing set\n");
-    else {
-	for (i = 0; i < num_debug; i++) {
-	    if (debug_map[i].bit != 0 &&
-	        (pmDebug & debug_map[i].bit) != 0) {
-		nset++;
-		if (nset > 1)
-		    fputc(',', f);
-		fprintf(f, "%s", debug_map[i].name);
-	    }
-	}
-	fputc('\n', f);
-    }
-
     nset = 0;
     fprintf(f, "pmDebugOptions:\t");
     for (i = 0; i < num_debug; i++) {
@@ -1711,55 +1700,74 @@ __pmDumpDebug(FILE *f)
 	fputc('\n', f);
 }
 
-/*
- * old routine for backwards compatibility ...
- *	return 32-bit bit debug flags
- */
-int
-__pmParseDebug(const char *spec)
-{
-    if (pmDebugOptions.deprecated)
-	fprintf(stderr, "Warning: deprecated __pmParseDebug() called\n");
-    return debug(spec, DEBUG_SET, DEBUG_OLD);
-}
-
 /* new routine to set debug options */
 int
 pmSetDebug(const char *spec)
 {
-    return debug(spec, DEBUG_SET, DEBUG_NEW);
+    return debug(spec, DEBUG_SET);
 }
 
 /* new routine to clear debug options */
 int
 pmClearDebug(const char *spec)
 {
-    return debug(spec, DEBUG_CLEAR, DEBUG_NEW);
+    return debug(spec, DEBUG_CLEAR);
+}
+
+/* new routine to test if one or more debug options is set */
+int
+pmDebug(const char *spec)
+{
+    return debug(spec, DEBUG_TEST);
 }
 
 /*
- * Interface for setting debugging options by bit-field (deprecated) rather
- * than by name (new scheme).
- * This routine is used by PMDAs that have a control metric that maps onto
- * pmDebug, e.g. sample.control or trace.control
- * For symmetry with pmSetDebug() the effects are additive, so a PMDA
- * that used to pmDebug = value now needs to pmClearDebug("all") and then
- * __pmSetDebugBits(value).
+ * return debug options that are currently set as a comma-separated
+ * list
  */
-void
-__pmSetDebugBits(int value)
+
+char *
+pmGetDebug(void)
 {
     int		i;
+    int		first = 1;
+    size_t	retlen;
+    char	*ans;
+    int		all = 1;
 
-    if (pmDebugOptions.deprecated)
-	fprintf(stderr, "Warning: deprecated __pmSetDebugBits() called\n");
-
+    retlen = 1;
     for (i = 0; i < num_debug; i++) {
-	if (value & debug_map[i].bit) {
-	    /* this option has a bit-field equivalent that is set in value */
-	    pmSetDebug(debug_map[i].name);
+	if (*(debug_map[i].options)) {
+	    retlen += strlen(debug_map[i].name) + 1;
+	}
+	else
+	    all = 0;
+    }
+    if (all)
+	retlen = 4;	/* enough for "all" */
+    if ((ans = (char *)malloc(retlen)) == NULL) {
+	pmNoMem("pmGetDebug", retlen, PM_RECOV_ERR);
+	return NULL;
+    }
+    if (all) {
+	pmstrncpy(ans, retlen, "all");
+	return ans;
+    }
+    ans[0] = '\0';
+    if (retlen == 1) {
+	/* nothing set */
+	return ans;
+    }
+    for (i = 0; i < num_debug; i++) {
+	if (*(debug_map[i].options)) {
+	    if (!first)
+		pmstrncat(ans, retlen, ",");
+	    pmstrncat(ans, retlen, debug_map[i].name);
+	    if (first)
+		first = 0;
 	}
     }
+    return ans;
 }
 
 int
@@ -2075,8 +2083,7 @@ __pmSetClientId(const char *id)
 		    osstrerror_r(errmsg, sizeof(errmsg)));
 	}
 	else {
-	    strncpy(host, servInfoName, sizeof(host));
-	    host[sizeof(host) - 1] = '\0';
+	    pmstrncpy(host, sizeof(host), servInfoName);
 	    free(servInfoName);
 	}
 	vblen = strlen(host) + strlen(id) + 2;
@@ -2218,8 +2225,7 @@ __pmMakePath(const char *dir, mode_t mode)
     if (sts < 0 && oserror() != ENOENT)
 	return -1;
 
-    strncpy(path, dir, sizeof(path));
-    path[sizeof(path)-1] = '\0';
+    pmstrncpy(path, sizeof(path), dir);
 
     for (p = path+1; *p != '\0'; p++) {
 	if (*p == pmPathSeparator()) {
@@ -2278,8 +2284,7 @@ strndup(const char *s, size_t n)
     char	*buf;
 
     if ((buf = malloc(n + 1)) != NULL) {
-	strncpy(buf, s, n);
-	buf[n] = '\0';
+	pmstrncpy(buf, n, s);
     }
     return buf;
 }
@@ -2878,8 +2883,18 @@ __pmSetSignalHandler(int sig, __pmSignalHandler func)
     return 0;
 }
 
+/*
+ * need wrapper so real routine can be called from win32.c
+ */
 void
 pmSetProgname(const char *program)
+{
+    __pmSetProgname(program);
+}
+
+/* real pmSetName */
+void
+__pmSetProgname(const char *program)
 {
     char	*p;
 
@@ -2887,13 +2902,13 @@ pmSetProgname(const char *program)
 
     if (program == NULL) {
 	/* Restore the default application name */
-	pmProgname = "pcp";
+	myprogname = "pcp";
     } else {
 	/* Trim command name of leading directory components */
-	pmProgname = (char *)program;
-	for (p = pmProgname; *p; p++)
+	myprogname = (char *)program;
+	for (p = myprogname; *p; p++)
 	    if (*p == '/')
-		pmProgname = p+1;
+		myprogname = p+1;
     }
 }
 
