@@ -151,20 +151,21 @@ pmSetProcessIdentity(const char *username)
 void
 pmSetProgname(const char *program)
 {
-    char	*p, *suffix = NULL;
     static int	setup;
     WORD	wVersionRequested = MAKEWORD(2, 2);
     WSADATA	wsaData;
 
     if (program == NULL) {
 	/* Restore the default application name */
-	pmProgname = "pcp";
+	__pmSetProgname(NULL);
     } else {
 	/* Trim command name of leading directory components */
-	pmProgname = (char *)program;
-	for (p = pmProgname; *p; p++) {
+	char	*p, *start, *suffix = NULL;
+	char	*dup;
+	dup = strdup(program);
+	for (p = start = dup; *p; p++) {
 	    if (*p == '\\' || *p == '/') {
-		pmProgname = p + 1;
+		start = p + 1;
 		suffix = NULL;
 	    }
 	    else if (*p == '.')
@@ -173,6 +174,8 @@ pmSetProgname(const char *program)
 	/* Drop the .exe suffix from the name if we found it */
 	if (suffix && strcmp(suffix, ".exe") == 0)
 	    *suffix = '\0';
+	__pmSetProgname(start);
+	free(dup);
     }
 
     if (setup)
@@ -1047,6 +1050,7 @@ int
 setenv(const char *name, const char *value, int overwrite)
 {
     char	*ebuf;
+    size_t	buf_size;
 
     if (getenv(name) != NULL) {		/* THREADSAFE */
 	/* already in the environment */
@@ -1054,12 +1058,14 @@ setenv(const char *name, const char *value, int overwrite)
 	    return(0);
     }
 
-    if ((ebuf = (char *)malloc(strlen(name) + strlen(value) + 2)) == NULL)
+    /* +1 for = and +1 for \0 */
+    buf_size = strlen(name) + strlen(value) + 2;
+    if ((ebuf = (char *)malloc(buf_size)) == NULL)
 	return -1;
 
-    strncpy(ebuf, name, strlen(name)+1);
-    strncat(ebuf, "=", 1);
-    strncat(ebuf, value, strlen(value));
+    pmstrncpy(ebuf, buf_size, name);
+    pmstrncat(ebuf, buf_size, "=");
+    pmstrncat(ebuf, buf_size, value);
 
     return _putenv(ebuf);		/* THREADSAFE */
 }
@@ -1071,14 +1077,17 @@ int
 unsetenv(const char *name)
 {
     char	*ebuf;
+    size_t	buf_size;
     int		sts;
 
-    if ((ebuf = (char *)malloc(strlen(name) + 2)) == NULL)
+    /* +1 for = and +1 for \0 */
+    buf_size = strlen(name) + strlen(value) + 2;
+    if ((ebuf = (char *)malloc(buf_size)) == NULL)
 	return -1;
 
     /* strange but true */
-    strncpy(ebuf, name, strlen(name)+1);
-    strncat(ebuf, "=", 1);
+    pmstrncpy(ebuf, buf_size, name);
+    pmstrncat(ebuf, buf_size, "=");
     sts = _putenv(ebuf);		/* THREADSAFE */
     free(ebuf);
     return sts;
