@@ -10,17 +10,12 @@
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
  * License for more details.
- * 
- * You should have received a copy of the GNU Lesser General Public License
- * along with this library; if not, write to the Free Software Foundation,
- * Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA.
  */
 
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include <unistd.h>
-#include <errno.h>
 #include "trace.h"
 #include "trace_dev.h"
 
@@ -30,7 +25,7 @@ typedef struct bufctl {
     int			bc_size;
     int			bc_pincnt;
     char		*bc_buf;
-    char		*bc_bufend;
+    char		*bc_bufend;	/* last byte in bc_buf[] */
 } bufctl_t;
 
 static bufctl_t	*buf_free;
@@ -78,7 +73,7 @@ __pmtracefindPDUbuf(int need)
 	    return NULL;
 	}
 	pcp->bc_next = buf_free;
-	pcp->bc_bufend = &pcp->bc_buf[pcp->bc_size];
+	pcp->bc_bufend = &pcp->bc_buf[pcp->bc_size-1];
 	buf_free = pcp;
     }
 
@@ -99,7 +94,7 @@ __pmtracepinPDUbuf(void *handle)
     bufctl_t	*prior = NULL;
 
     for (pcp = buf_free; pcp != NULL; pcp = pcp->bc_next) {
-	if (pcp->bc_buf <= (char *)handle && (char *)handle < pcp->bc_bufend)
+	if (pcp->bc_buf <= (char *)handle && (char *)handle <= pcp->bc_bufend)
 	    break;
 	prior = pcp;
     }
@@ -120,7 +115,7 @@ __pmtracepinPDUbuf(void *handle)
     }
     else {
 	for (pcp = buf_pin; pcp != NULL; pcp = pcp->bc_next) {
-	    if (pcp->bc_buf <= (char *)handle && (char *)handle < pcp->bc_bufend)
+	    if (pcp->bc_buf <= (char *)handle && (char *)handle <= pcp->bc_bufend)
 		break;
 	}
 	if (pcp != NULL)
