@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2014,2016-2017,2020-2021 Red Hat.
+ * Copyright (c) 2014,2016-2017,2020-2021,2025 Red Hat.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -12,6 +12,7 @@
  * Library General Public License for more details.
  */
 
+#include <assert.h>
 #include <ctype.h>
 #include "pmapi.h"
 #include "libpcp.h"
@@ -27,6 +28,7 @@
 #define UNIX			"unix"
 #define LOCATION		"location"
 #define CONTENT_TYPE		"content-type"
+#define CONTENT_LENGTH		"content-length"
 
 static int
 http_client_connectunix(const char *path, struct timeval *timeout)
@@ -223,11 +225,11 @@ static int
 http_client_connect(http_client *cp)
 {
     http_parser_url	*up = &cp->parser_url;
-    const char		*protocol, *url = cp->url;
+    const char		*protocol, *url = cp->conn;
     size_t		length;
 
     if (pmDebugOptions.http)
-	fprintf(stderr, "http_client_connect fd=%d\n", cp->fd);
+	fprintf(stderr, "%s fd=%d\n", __FUNCTION__, cp->fd);
 
     if (cp->fd != -1)	/* already connected */
 	return 0;
@@ -241,16 +243,16 @@ http_client_connect(http_client *cp)
 
 	if (!up->field_data[UF_HOST].len) {
 	    cp->error_code = -EINVAL;
-	    return -1;
+	    return cp->error_code;
 	}
 	length = 1;	/* just the terminator here */
 	length += up->field_data[UF_HOST].len;
 	if (length > MAXHOSTNAMELEN) {
 	    cp->error_code = -EINVAL;
-	    return -1;
+	    return cp->error_code;
 	}
 	length = up->field_data[UF_HOST].len;
-	strncpy(host, url + up->field_data[UF_HOST].off, length);
+	memcpy(host, url + up->field_data[UF_HOST].off, length);
 	host[length] = '\0';
 	port = up->port ? up->port : HTTP_PORT;
 
@@ -263,14 +265,14 @@ http_client_connect(http_client *cp)
 
 	if (!up->field_data[UF_HOST].len || !up->field_data[UF_PATH].len) {
 	    cp->error_code = -EINVAL;
-	    return -1;
+	    return cp->error_code;
 	}
 	length = 3;	/* 2 separators, terminator */
 	length += up->field_data[UF_HOST].len;
 	length += up->field_data[UF_PATH].len;
 	if (length > MAXPATHLEN) {
 	    cp->error_code = -EINVAL;
-	    return -1;
+	    return cp->error_code;
 	}
 	pmsprintf(path, sizeof(path), "/%.*s/%.*s",
 		up->field_data[UF_HOST].len, url + up->field_data[UF_HOST].off,
@@ -301,46 +303,39 @@ http_client_get(http_client *cp)
 {
     char		buf[BUFSIZ];
     char		host[MAXHOSTNAMELEN];
-    char		*bp = &buf[0], *url = cp->url;
+    char		*bp = &buf[0], *url = cp->conn;
     http_parser_url	*up = &cp->parser_url;
     const char		*path, *agent, *version, *protocol;
-    size_t		hostlen, len = 0, length;
+    size_t		len = 0, length;
     int			sts;
 
-
     /* sanitize request parameters */
     if ((agent = cp->user_agent) == NULL)
 	agent = pmGetProgname();
     if ((version = cp->agent_vers) == NULL)
 	version = "1.0";
-    if ((path = url + up->field_data[UF_PATH].off) == NULL ||
-	up->field_data[UF_PATH].off == 0 ||
-	strchr(path, '/') == NULL) {
-	path = "/";	/* assume root-level request */
+    if ((path = cp->path) == NULL) {
+	if ((path = url + up->field_data[UF_PATH].off) == NULL ||
+	    up->field_data[UF_PATH].off == 0 ||
+	    strchr(path, '/') == NULL)
+	    path = "/";	/* assume root-level request */
     }
-    hostlen = up->field_data[UF_HOST].len;
-    strncpy(host, url + up->field_data[UF_HOST].off, hostlen);
-    host[hostlen] = '\0';
-#if 0
-    //    strncpy(host, "localhost", sizeof("localhost"));
-    //    host[sizeof("localhost")] = '\0';
-    //    strncpy(path, "/containers/8d70f8a47a6b6e515fb8e40d31da7928de70e883c235ba16b132e6a3b4f8267d/json", sizeof("/containers/8d70f8a47a6b6e515fb8e40d31da7928de70e883c235ba16b132e6a3b4f8267d/json"));
-    //    pmNotifyErr(LOG_DEBUG, "hit here: %s", cp->type_buffer);
-#endif
-    
+
     protocol = url + up->field_data[UF_SCHEMA].off;
     length = up->field_data[UF_SCHEMA].len;
+    if (length == sizeof(UNIX)-1 && strncmp(protocol, UNIX, length) == 0) {
+	assert(sizeof(host) > 10);
+	strcpy(host, "localhost");
+    } else {
+	length = up->field_data[UF_HOST].len;
+	memcpy(host, url + up->field_data[UF_HOST].off, length);
+	host[length] = '\0';
+    }
+
     /* prepare and send a GET request */
-    if (length == sizeof(HTTP)-1 && strncmp(protocol, UNIX, length) == 0) {
-	len += pmsprintf(bp+len, sizeof(buf)-len, "GET %s HTTP/%s\r\n",
-			cp->type_buffer, http_versionstr(cp->http_version));
-	len += pmsprintf(bp+len, sizeof(buf)-len, "Host: %s\r\n", "localhost");
-    }
-    else {
-	len += pmsprintf(bp+len, sizeof(buf)-len, "GET %s HTTP/%s\r\n",
+    len += pmsprintf(bp+len, sizeof(buf)-len, "GET %s HTTP/%s\r\n",
 			path, http_versionstr(cp->http_version));
-	len += pmsprintf(bp+len, sizeof(buf)-len, "Host: %s\r\n", host);
-    }
+    len += pmsprintf(bp+len, sizeof(buf)-len, "Host: %s\r\n", host);
     len += pmsprintf(bp+len, sizeof(buf)-len, "User-Agent: %s/%s\r\n",
 			agent, version);
     /* establish persistent connections (default in HTTP/1.1 onward) */
@@ -357,7 +352,6 @@ http_client_get(http_client *cp)
 	    sts = 1;
 	} else {
 	    cp->error_code = sts;
-	    sts = -1;
 	}
 	http_client_disconnect(cp);
     } else {
@@ -365,7 +359,77 @@ http_client_get(http_client *cp)
     }
 
     if (pmDebugOptions.http)
-	fprintf(stderr, "http_client_get sts=%d\n", sts);
+	fprintf(stderr, "%s sts=%d\n", __FUNCTION__, sts);
+
+    return sts;
+}
+
+static int
+http_client_post(http_client *cp)
+{
+    char		buf[BUFSIZ];
+    char		host[MAXHOSTNAMELEN];
+    char		*bp = &buf[0], *url = cp->conn;
+    http_parser_url	*up = &cp->parser_url;
+    const char		*path, *agent, *version, *protocol;
+    size_t		len = 0, length;
+    int			sts;
+
+    /* sanitize request parameters */
+    if ((agent = cp->user_agent) == NULL)
+	agent = pmGetProgname();
+    if ((version = cp->agent_vers) == NULL)
+	version = "1.0";
+    if ((path = cp->path) == NULL) {
+	if ((path = url + up->field_data[UF_PATH].off) == NULL ||
+	    up->field_data[UF_PATH].off == 0 ||
+	    strchr(path, '/') == NULL)
+	    path = "/";	/* assume root-level request */
+    }
+
+    protocol = url + up->field_data[UF_SCHEMA].off;
+    length = up->field_data[UF_SCHEMA].len;
+    if (length == sizeof(UNIX)-1 && strncmp(protocol, UNIX, length) == 0) {
+	assert(sizeof(host) > 10);
+	strcpy(host, "localhost");
+    } else {
+	length = up->field_data[UF_HOST].len;
+	memcpy(host, url + up->field_data[UF_HOST].off, length);
+	host[length] = '\0';
+    }
+
+    /* prepare and send a POST request */
+    len += pmsprintf(bp+len, sizeof(buf)-len, "POST %s HTTP/%s\r\n",
+			path, http_versionstr(cp->http_version));
+    len += pmsprintf(bp+len, sizeof(buf)-len, "Host: %s\r\n", host);
+    len += pmsprintf(bp+len, sizeof(buf)-len, "User-Agent: %s/%s\r\n",
+			agent, version);
+    len += pmsprintf(bp+len, sizeof(buf)-len, "%s: %s\r\n",
+		    	CONTENT_TYPE, cp->content_type);
+    len += pmsprintf(bp+len, sizeof(buf)-len, "%s: %zu\r\n",
+		    	CONTENT_LENGTH, cp->input_length);
+    /* establish persistent connections (default in HTTP/1.1 onward) */
+    if (cp->http_version < PV_HTTP_1_1)
+	len += pmsprintf(bp+len, sizeof(buf)-len, "Connection: keep-alive\r\n");
+    len += pmsprintf(bp+len, sizeof(buf)-len, "\r\n");
+    buf[BUFSIZ-1] = '\0';
+
+    if (pmDebugOptions.http && pmDebugOptions.desperate)
+	fprintf(stderr, "Sending HTTP POST:\n\n%s\n", buf);
+
+    if ((sts = __pmSend(cp->fd, buf, len, 0)) < 0 || /* header then body */
+	(sts = __pmSend(cp->fd, cp->input_buffer, cp->input_length, 0)) < 0) {
+	if (__pmSocketClosed())
+	    sts = 1;
+	else
+	    cp->error_code = sts;
+	http_client_disconnect(cp);
+    } else {
+	sts = 0;
+    }
+
+    if (pmDebugOptions.http)
+	fprintf(stderr, "%s sts=%d\n", __FUNCTION__, sts);
 
     return sts;
 }
@@ -431,13 +495,16 @@ reset_url_location(const char *tourl, size_t tolen, http_parser_url *top,
 	size = bytes + length + 2;	/* separator + terminator */
 	if ((url = malloc(size)) == NULL)
 	    return -ENOMEM;
-	strncpy(url, curl, bytes);
+	memcpy(url, curl, bytes);
 	str = url + bytes;
 	if (*suffix != '/')
 	    *str++ = '/';
-	strncat(str, suffix, length);
+	memcpy(str, suffix, length);
 	url[size - 1] = '\0';
-	http_parser_parse_url(url, size, 0, fromp);
+	if (http_parser_parse_url(url, size, 0, fromp) != 0) {
+	    free(url);
+	    return -ENOMEM;
+	}
 
 	if (pmDebugOptions.http)
 	    fprintf(stderr, "Redirecting from '%s' to '%s'\n", curl, url);
@@ -471,6 +538,7 @@ static int
 on_header_value(http_parser *pp, const char *offset, size_t length)
 {
     http_client		*cp = (http_client *)pp->data;
+    char		*tp;
     int			sts;
 
     if (pmDebugOptions.http)
@@ -483,10 +551,12 @@ on_header_value(http_parser *pp, const char *offset, size_t length)
 	    http_parser_url	up;
 
 	    http_parser_url_init(&up);
-	    if ((sts = http_parser_parse_url(offset, length, 0, &up)) != 0)
-		return sts;
+	    if (http_parser_parse_url(offset, length, 0, &up) != 0) {
+		cp->error_code = -EINVAL;
+		return 1;
+	    }
 	    if ((sts = reset_url_location(offset, length, &up,
-					&cp->url, &cp->parser_url)) < 0) {
+					&cp->conn, &cp->parser_url)) < 0) {
 		cp->error_code = -ENOMEM;
 		return 1;
 	    }
@@ -497,14 +567,16 @@ on_header_value(http_parser *pp, const char *offset, size_t length)
     if (cp->flags & F_CONTENT_TYPE) {	/* stash content-type */
 	cp->flags &= ~F_CONTENT_TYPE;
 
-	if (cp->type_length > 0) {
+	if (cp->type_bufferp) {
 	    if (length + 1 > cp->type_length) {
-		cp->error_code = -E2BIG;
-		if (pmDebugOptions.http)
-		    fprintf(stderr, "on_header_value: Setting error E2BIG\n");
-		return 1;
+		if ((tp = realloc(cp->type_buffer, length + 1)) == NULL) {
+		    cp->error_code = -ENOMEM;
+		    return 1;
+		}
+		*cp->type_bufferp = cp->type_buffer = tp;
+		*cp->type_lengthp = cp->type_length = length;
 	    }
-	    strncpy(cp->type_buffer, offset, length);
+	    memcpy(cp->type_buffer, offset, length);
 	    cp->type_buffer[length] = '\0';
 	}
     }
@@ -516,18 +588,24 @@ static int
 on_body(http_parser *pp, const char *offset, size_t length)
 {
     http_client		*cp = (http_client *)pp->data;
+    char		*bp;
 
     if (pmDebugOptions.http)
 	fprintf(stderr, "Body: %.*s\n", (int)length, offset);
 
-    if (length > cp->body_length - cp->offset) {
-	cp->error_code = -E2BIG;
-	if (pmDebugOptions.http)
-	    fprintf(stderr, "on_body: Setting error E2BIG\n");
-	return 1;
+    if (length > 0 && length + 1 > cp->body_length - cp->offset) {
+	if ((bp = realloc(cp->body_buffer, cp->offset + length + 1)) == NULL) {
+            cp->error_code = -ENOMEM;
+	    return 1;
+	}
+	*cp->body_bufferp = cp->body_buffer = bp;
+	*cp->body_lengthp = cp->body_length = cp->offset + length;
+    }
+    if (length > 0) {
+	memcpy(cp->body_buffer + cp->offset, offset, length);
+	cp->offset += length;
+	cp->body_buffer[cp->offset] = '\0';
     }
-    strncpy(cp->body_buffer + cp->offset, offset, length);
-    cp->offset += length;
     return 0;
 }
 
@@ -536,6 +614,7 @@ on_message_complete(http_parser *pp)
 {
     http_client		*cp = (http_client *)pp->data;
 
+    cp->status_code = pp->status_code;
     cp->flags |= F_MESSAGE_END;
     return 0;
 }
@@ -578,10 +657,11 @@ http_client_response(http_client *cp)
     }
 
     if (pmDebugOptions.http)
-	fprintf(stderr, "http_client_response\n");
+	fprintf(stderr, "%s\n", __FUNCTION__);
 
     http_parser_init(&cp->parser, HTTP_RESPONSE);
     cp->parser.data = (void *)cp;
+    cp->status_code = 0;
     cp->error_code = 0;
     cp->offset = 0;
 
@@ -625,9 +705,9 @@ http_client_response(http_client *cp)
     }
 
     if (pmDebugOptions.http)
-	fprintf(stderr, "returning offset=%zu\n", cp->offset);
+	fprintf(stderr, "returning http status=%d\n", cp->status_code);
 
-    return cp->offset;
+    return cp->status_code;
 }
 
 http_client *
@@ -650,7 +730,9 @@ void
 pmhttpFreeClient(http_client *cp)
 {
     http_client_disconnect(cp);
-    free(cp->url);
+    free(cp->body_buffer);
+    free(cp->type_buffer);
+    free(cp->conn);
     free(cp);
 }
 
@@ -696,7 +778,7 @@ http_compare_source(http_parser_url *a, const char *urla,
 	if (a->port != b->port)
 	    return 1;
 	if (strncmp(urla + a->field_data[UF_SCHEMA].off,
-		    urlb + b->field_data[UF_SCHEMA].off, length) == 0)
+		    urlb + b->field_data[UF_SCHEMA].off, length) != 0)
 	    return 1;
 	return 0;
     }
@@ -709,54 +791,80 @@ http_compare_source(http_parser_url *a, const char *urla,
 }
 
 static int
-http_client_prepare(http_client *cp, const char *url,
-		char *body_buffer, size_t body_length,
-		char *type_buffer, size_t type_length)
+http_client_prepare(http_client *cp, const char *url, /* conn */
+		const char *path, const char *in_content_type,
+		const void *in_body_buffer, size_t in_body_length,
+		char **out_body_buffer, size_t *out_body_length,
+		char **out_type_buffer, size_t *out_type_length)
 {
     http_parser_url	parser_url;
     char		*new_url;
     int			sts;
 
-    cp->body_buffer = body_buffer;
-    cp->body_length = body_length;
-    cp->type_buffer = type_buffer;
-    cp->type_length = type_length;
+    cp->body_bufferp = out_body_buffer;
+    cp->body_lengthp = out_body_length;
+    cp->body_buffer = out_body_buffer ? *out_body_buffer : NULL;
+    cp->body_length = out_body_length ? *out_body_length : 0;
+    cp->offset = 0;
+
+    cp->type_bufferp = out_type_buffer;
+    cp->type_lengthp = out_type_length;
+    cp->type_buffer = out_type_buffer ? *out_type_buffer : NULL;
+    cp->type_length = out_type_length ? *out_type_length : 0;
+
+    cp->path = path;
+    cp->content_type = in_content_type;
+    cp->input_buffer = in_body_buffer;
+    cp->input_length = in_body_length;
+
+    cp->flags &= ~F_MESSAGE_END;
 
     /* extract individual fields from the given URL */
     http_parser_url_init(&parser_url);
     if ((sts = http_parser_parse_url(url, strlen(url), 0, &parser_url)) != 0) {
-	cp->error_code = sts;
-	return -1;
+	cp->error_code = -EINVAL;
+	return cp->error_code;
     }
 
     /* short-circuit if we are making a request from a connected server */
-    if (http_compare_source(&parser_url, url, &cp->parser_url, cp->url) == 0)
-	return 0;
-    http_client_disconnect(cp);
+    if (http_compare_source(&parser_url, url, &cp->parser_url, cp->conn) != 0) {
+        http_client_disconnect(cp);
+    }
 
     if ((new_url = strdup(url)) == NULL) {
 	cp->error_code = -ENOMEM;
-	return -1;
+	return cp->error_code;
     }
-    free(cp->url);
-    cp->url = new_url;
+    free(cp->conn);
+    cp->conn = new_url;
     cp->parser_url = parser_url;
     return 0;
 }
 
+/* HTTP GET via fixed-size output buffers (retired interface) */
 int
 pmhttpClientFetch(http_client *cp, const char *url,
 		char *body_buffer, size_t body_length,
 		char *type_buffer, size_t type_length)
+{
+    return -ENOTSUP;
+}
+
+/* HTTP GET interface using dynamically-sized output buffers */
+int
+pmhttpClientGet(http_client *cp, const char *conn, const char *path,
+		char **out_body_buffer, size_t *out_body_length,
+		char **out_type_buffer, size_t *out_type_length)
 {
     int		sts = 0, redirected = 0;
 
     if (pmDebugOptions.http)
-	fprintf(stderr, "pmhttpClientFetch: %s\n", url);
+	fprintf(stderr, "%s: %s (%s)\n", __FUNCTION__, conn, path);
 
-    if (http_client_prepare(cp, url, body_buffer, body_length,
-			    type_buffer, type_length) != 0)
-	return -1;
+    if (http_client_prepare(cp, conn, path, NULL, NULL, 0,
+			    out_body_buffer, out_body_length,
+			    out_type_buffer, out_type_length) < 0)
+	return cp->error_code;
 
     while (redirected <= cp->max_redirect) {
 	/* ensure we're connected to the server */
@@ -770,6 +878,62 @@ pmhttpClientFetch(http_client *cp, const char *url,
 	    continue;
 
 	/* parse, extract body, handle redirect */
+	if ((sts = http_client_response(cp)) < 0) {
+	    if (sts == -EAGAIN) {	/* server closed */
+		if (pmDebugOptions.http)
+		    fprintf(stderr, "Got server closed, trying again\n");
+		continue;
+	    }
+	    if (sts == -EMLINK) {	/* http redirect */
+		if (pmDebugOptions.http)
+		    fprintf(stderr, "Got redirect, trying again, attempt: %d\n", redirected);
+		redirected++;
+		continue;
+	    }
+	    if (pmDebugOptions.http)
+		fprintf(stderr, "Got error value: %d\n", sts);
+	    break;	/* propagate errors */
+	}
+	break;	/* successful exchange */
+    }
+
+    if (pmDebugOptions.http)
+	fprintf(stderr, "%s sts=%d\n", __FUNCTION__, sts);
+
+    return sts;
+}
+
+/* HTTP POST interface - fixed-sized inputs, dynamically-sized outputs */
+int
+pmhttpClientPost(http_client *cp, const char *conn, const char *path,
+		const void *in_body_buffer, size_t in_body_length,
+		const char *in_content_type,
+		char **out_body_buffer, size_t *out_body_length,
+		char **out_type_buffer, size_t *out_type_length)
+{
+    int		sts = 0, redirected = 0;
+
+    if (pmDebugOptions.http)
+	fprintf(stderr, "%s: %s (%s)\n", __FUNCTION__, conn, path);
+
+    if (http_client_prepare(cp, conn, path, in_content_type,
+			    in_body_buffer, in_body_length,
+			    out_body_buffer, out_body_length,
+			    out_type_buffer, out_type_length) < 0)
+	return cp->error_code;
+
+    while (redirected <= cp->max_redirect) {
+	/* ensure we're connected to the server */
+	if ((sts = http_client_connect(cp)) < 0)
+	    return sts;
+
+	/* make a POST request via the given URL */
+	if ((sts = http_client_post(cp)) < 0)
+	    return sts;
+	if (sts == 1)	/* server connection is lost */
+	    continue;
+
+	/* parse, extract result, handle redirect */
 	if ((sts = http_client_response(cp)) < 0) {
 	    if (sts == -EAGAIN) {		/* server closed */
 		if (pmDebugOptions.http)
@@ -790,7 +954,7 @@ pmhttpClientFetch(http_client *cp, const char *url,
     }
 
     if (pmDebugOptions.http)
-	fprintf(stderr, "pmhttpClientFetch sts=%d\n", sts);
+	fprintf(stderr, "%s sts=%d\n", __FUNCTION__, sts);
 
     return sts;
 }
