@@ -88,7 +88,7 @@ load_prepare_metric(const char *name, void *arg)
     char		pmmsg[PM_MAXERRMSGLEN];
     char		*hname;
     pmID		pmid;
-    sds			msg;
+    sds			msg = NULL;
     int			sts;
 
     if ((sts = pmLookupName(1, &name, &pmid)) < 0) {
@@ -268,7 +268,7 @@ pmwebapi_extract_events(pmValueSet *vsp, int inst)
 {
     sds			s;
     int			record, param, nrecords, flags, sts;
-    pmResult		**results, *result;
+    pmResult_v2		**results, *result;
 
     if ((sts = nrecords = pmUnpackEventRecords(vsp, inst, &results)) < 0) {
 	if (pmDebugOptions.series)
@@ -298,7 +298,7 @@ pmwebapi_extract_highres_events(pmValueSet *vsp, int inst)
 {
     sds			s;
     int			record, param, nrecords, flags, sts;
-    pmHighResResult	**results, *result;
+    pmResult	**results, *result;
 
     if ((sts = pmUnpackHighResEventRecords(vsp, inst, &results)) < 0) {
 	if (pmDebugOptions.series)
@@ -440,7 +440,7 @@ series_cache_update(seriesLoadBaton *baton, struct dict *exclude)
 {
     seriesGetContext	*context = &baton->pmapi;
     context_t		*cp = &context->context;
-    pmHighResResult	*result = context->result;
+    pmResult		*result = context->result;
     pmValueSet		*vsp;
     metric_t		*metric = NULL;
     char		ts[64];
@@ -456,7 +456,7 @@ series_cache_update(seriesLoadBaton *baton, struct dict *exclude)
 	goto out;
     }
 
-    pmSortHighResInstances(result);
+    pmSortInstances(result);
 
     for (i = 0; i < result->numpmid; i++) {
 	vsp = result->vset[i];
@@ -512,14 +512,14 @@ static int
 server_cache_series(seriesLoadBaton *baton)
 {
     char		pmmsg[PM_MAXERRMSGLEN];
-    sds			msg;
+    sds			msg = NULL;
     int			sts;
 
     if (baton->pmapi.context.type != PM_CONTEXT_ARCHIVE)
 	return -ENOTSUP;
 
-    if ((sts = pmSetModeHighRes(PM_MODE_FORW, &baton->timing.start, NULL)) < 0) {
-	infofmt(msg, "pmSetModeHighRes failed: %s",
+    if ((sts = pmSetMode(PM_MODE_FORW, &baton->timing.start, NULL)) < 0) {
+	infofmt(msg, "pmSetMode failed: %s",
 		pmErrStr_r(sts, pmmsg, sizeof(pmmsg)));
 	batoninfo(baton, PMLOG_ERROR, msg);
 	return sts;
@@ -550,7 +550,7 @@ server_cache_update_done(void *arg)
     seriesGetContext	*context = &baton->pmapi;
 
     /* finish book-keeping for the current record */
-    pmFreeHighResResult(context->result);
+    pmFreeResult(context->result);
     context->result = NULL;
     context->count++;
     context->done = NULL;
@@ -566,12 +566,12 @@ fetch_archive(uv_work_t *req)
     seriesLoadBaton	*baton = (seriesLoadBaton *)req->data;
     seriesGetContext	*context = &baton->pmapi;
     context_t		*cp = &context->context;
-    pmHighResResult	*result;
+    pmResult		*result;
     int			sts;
 
     assert(context->result == NULL);
     if ((context->error = sts = pmUseContext(cp->context)) >= 0)
-	if ((context->error = sts = pmFetchHighResArchive(&result)) >= 0)
+	if ((context->error = sts = pmFetchArchive(&result)) >= 0)
 	    context->result = result;
 }
 
@@ -600,7 +600,7 @@ fetch_archive_done(uv_work_t *req, int status)
 	    if (pmDebugOptions.series)
 		fprintf(stderr, "%s: time window end\n", "fetch_archive_done");
 	    sts = PM_ERR_EOL;
-	    pmFreeHighResResult(context->result);
+	    pmFreeResult(context->result);
 	    context->result = NULL;
 	}
     }
@@ -734,7 +734,7 @@ load_prepare_exclude_metric(const char *name, void *arg)
 {
     seriesLoadBaton	*baton = (seriesLoadBaton *)arg;
     char		pmmsg[PM_MAXERRMSGLEN];
-    sds			msg;
+    sds			msg = NULL;
     pmID		pmid;
     int			i;
     int			sts;
@@ -763,7 +763,7 @@ load_prepare_included_metrics(seriesLoadBaton *baton)
     context_t		*cp = &baton->pmapi.context;
     const char		**metrics = baton->metrics;
     char		pmmsg[PM_MAXERRMSGLEN];
-    sds			msg;
+    sds			msg = NULL;
     int			i, sts;
 
     for (i = 0; i < baton->nmetrics; i++) {
@@ -783,7 +783,7 @@ load_prepare_excluded_metrics(seriesLoadBaton *baton)
     pmSeriesModule	*module = (pmSeriesModule *)baton->module;
     seriesModuleData	*data = getSeriesModuleData(module);
     char		pmmsg[PM_MAXERRMSGLEN];
-    sds			msg;
+    sds			msg = NULL;
     int			i, sts;
     sds			exclude_metrics_option;
     sds			*patterns = NULL;
@@ -983,7 +983,7 @@ doneSeriesGetContext(seriesGetContext *context, const char *caller)
 
     if (context->error && !context->loaded) {
 	char		pmmsg[PM_MAXERRMSGLEN];
-	sds		msg;
+	sds		msg = NULL;
 
 	if (context->error == PM_ERR_EOL) {
 	    context->loaded = 1;
@@ -1177,7 +1177,7 @@ series_load(pmSeriesSettings *settings,
 {
     seriesLoadBaton	*baton;
     seriesModuleData	*data = getSeriesModuleData(&settings->module);
-    sds			msg;
+    sds			msg = NULL;
     int			i;
 
     if (data == NULL)
@@ -1249,7 +1249,7 @@ pmSeriesDiscoverSource(pmDiscoverEvent *event, void *arg)
     discoverModuleData	*data = getDiscoverModuleData(module);
     seriesLoadBaton	*baton;
     context_t		*cp;
-    sds			msg;
+    sds			msg = NULL;
     int			i;
 
     if (data == NULL || data->slots == NULL || data->slots->state != SLOTS_READY)
@@ -1330,7 +1330,7 @@ pmSeriesDiscoverLabels(pmDiscoverEvent *event,
     struct indom	*indom;
     struct instance	*instance;
     char		errmsg[PM_MAXERRMSGLEN], idbuf[64];
-    sds			msg;
+    sds			msg = NULL;
     int			i, id;
 
     if (baton == NULL || baton->slots == NULL || baton->slots->state != SLOTS_READY)
@@ -1463,7 +1463,7 @@ pmSeriesDiscoverMetric(pmDiscoverEvent *event,
     pmDiscover		*p = (pmDiscover *)event->data;
     seriesLoadBaton	*baton = p->baton;
     struct metric	*metric;
-    sds			msg;
+    sds			msg = NULL;
     int			i;
 
     if (pmDebugOptions.discovery) {
@@ -1484,7 +1484,7 @@ pmSeriesDiscoverMetric(pmDiscoverEvent *event,
 }
 
 void
-pmSeriesDiscoverValues(pmDiscoverEvent *event, pmHighResResult *result, void *arg)
+pmSeriesDiscoverValues(pmDiscoverEvent *event, pmResult *result, void *arg)
 {
     pmDiscoverModule	*module = event->module;
     pmDiscover		*p = (pmDiscover *)event->data;
@@ -1520,7 +1520,7 @@ pmSeriesDiscoverInDom(pmDiscoverEvent *event, pmInResult *in, void *arg)
     struct domain	*domain;
     struct indom	*indom;
     pmInDom		id = in->indom;
-    sds			msg;
+    sds			msg = NULL;
     int			i;
 
     if (pmDebugOptions.discovery)
