@@ -115,7 +115,8 @@ saroptions(void)
 	/*
 	** add generic flags
 	*/
-	return strncat(allflags, sarflags, pricnt+32 - 1);
+	pmstrncat(allflags, pricnt+32, sarflags);
+	return allflags;
 }
 
 int
@@ -247,10 +248,10 @@ atopsar(int argc, char *argv[])
 			char	*endnum, *arg;
 
 			arg = argv[opts.optind++];
-			if (pmParseInterval(arg, &opts.interval, &endnum) < 0)
+			if (pmParseHighResInterval(arg, &opts.interval, &endnum) < 0)
 			{
 				pmprintf(
-			"%s: %s option not in pmParseInterval(3) format:\n%s\n",
+			"%s: %s option not in pmParseHighResInterval(3) format:\n%s\n",
 					pmGetProgname(), arg, endnum);
 				free(endnum);
 				opts.errors++;
@@ -287,8 +288,9 @@ atopsar(int argc, char *argv[])
 	if (opts.samples > 0)
 		nsamples = opts.samples + 1;
 
-	if (opts.interval.tv_sec || opts.interval.tv_usec)
+	if (opts.interval.tv_sec || opts.interval.tv_nsec) {
 		interval = opts.interval;
+	}
 
 	/*
 	** if no report-flags have been specified, take the first
@@ -620,7 +622,7 @@ reportraw(double timenow, double numsecs,
 		/*
 		** initialize variables for new report
 		*/
-		pmtimevalFromReal(timenow, &pretime);
+		pmtimespecFromReal(timenow, &pretime);
 
 		curline   = 1;
 		headline  = 0;
@@ -648,7 +650,7 @@ reportraw(double timenow, double numsecs,
 		if (usecolors)
 			printf(COLSETHEAD);
 
-		timed = pmtimevalToReal(&pretime);
+		timed = pmtimespecToReal(&pretime);
 		printf("%s  ", convtime(timed, timebuf, sizeof(timebuf)-1));
 
 		(pridef[prinow].prihead)(os_vers, os_rel, os_sub);
@@ -708,7 +710,7 @@ reportraw(double timenow, double numsecs,
 			printf("......................... logging restarted "
 			       ".........................\n");
 		}
-		pmtimevalFromReal(timenow, &pretime);
+		pmtimespecFromReal(timenow, &pretime);
 		curline++;
 
 		/*
@@ -837,7 +839,7 @@ reportraw(double timenow, double numsecs,
 			cleanstop(1);
 	}
 
-	pmtimevalFromReal(timenow, &pretime);
+	pmtimespecFromReal(timenow, &pretime);
 
 	return '\0';
 }
