@@ -21,9 +21,13 @@
 ** See the GNU General Public License for more details.
 */
 
+#ifndef __PHOTOPROC__
+#define __PHOTOPROC__
+
 #define	PNAMLEN		15
 #define	CMDLEN		255
 #define	CGRLEN		64
+#define	UTSLEN		15
 
 /* 
 ** structure containing only relevant process-info extracted 
@@ -54,13 +58,14 @@ struct tstat {
 		int	nthrslpi;	/* # threads in state 'S'       */
 		int	nthrslpu;	/* # threads in state 'D'       */
 		int	nthrrun;	/* # threads in state 'R'       */
+		int	nthridle;	/* # threads in state 'I'       */
 
 		int	ctid;		/* OpenVZ container ID		*/
 		int	vpid;		/* OpenVZ virtual PID		*/
 
 		int	wasinactive;	/* boolean: task inactive	*/
 
-		char	container[16];	/* Docker container id (12 pos)	*/
+		char	utsname[UTSLEN+1];/* UTS name container or pod	*/
 		char	cgpath[CGRLEN];	/* cgroup v2 path name          */
 	} gen;
 
@@ -81,6 +86,8 @@ struct tstat {
 		char	wchan[16];	/* wait channel string    	*/
 		count_t	rundelay;	/* schedstat rundelay (nanosec)	*/
 		count_t	blkdelay;	/* blkio delay (ticks)		*/
+		count_t nvcsw;		/* voluntary cxt switch counts  */
+		count_t nivcsw;		/* involuntary csw counts       */
 	} cpu;
 
 	/* DISK STATISTICS						*/
@@ -166,7 +173,7 @@ struct devtstat {
 	unsigned long	nprocall;
 	unsigned long	nprocactive;
 
-        unsigned long   totrun, totslpi, totslpu, totzombie;
+        unsigned long   totrun, totslpi, totslpu, totidle, totzombie;
 };
 
 /*
@@ -189,3 +196,5 @@ void		deviattask(struct tstat *, unsigned long,
 int		getmaxpid(void);
 void		setup_photoproc(void);
 unsigned long	photoproc(struct tstat **, unsigned long *);
+
+#endif
