@@ -27,7 +27,7 @@ SamplingItem::SamplingItem(Chart *parent,
     pmDesc desc = mp->desc().desc();
 
     my.chart = parent;
-    my.info = QString::null;
+    my.info = QString();
 
     // initialize the pcp data and item data arrays
     my.dataCount = 0;
@@ -125,9 +125,9 @@ SamplingItem::updateValues(bool forward,
 
     // sz will be the number of previous samples we want to keep.
     if (my.dataCount < sampleHistory)
-	sz = qMax(0, (int)(my.dataCount * sizeof(double)));
+	sz = qMax(0, (int)(my.dataCount));
     else
-	sz = qMax(0, (int)((my.dataCount - 1) * sizeof(double)));
+	sz = qMax(0, (int)((my.dataCount - 1)));
 
     if (forward) {
 	// Keep sz samples and add the new sample to the beginning.
@@ -549,7 +549,7 @@ SamplingEngine::updateValues(bool forward,
     Chart::Style style = my.chart->my.style;
 
     // Drive new values into each chart item
-    for (int i = 0; i < itemCount; i++) {
+    for (i = 0; i < itemCount; i++) {
 	samplingItem(i)->updateValues(forward, my.rateConvert, &my.units,
 					size, points, left, right, delta);
     }
