@@ -150,6 +150,7 @@ main(int argc, char **argv)
 	    "Usage: %s [options] host:{a|B|E|e|f|h|s}N[/M[.I]] [...]\n\n", 
 	    pmGetProgname());
 	fputs("Options:\n"
+	      "  -D debug     set debug options, see pmdbg(1)\n"
 	      "  -d domain    use domain (numeric) for metrics domain of PMDA\n"
 	      "  -i port      expect PMCD to connect on given inet port (number or name)\n"
 	      "  -l logfile   redirect diagnostics and trace output to logfile\n"
@@ -331,10 +332,9 @@ main(int argc, char **argv)
 		if (mypasswd != NULL) {
 		    if (strcmp(cisco[i].passwd, mypasswd) != 0) {
 			fprintf(stderr, 
-				"%s: conflicting user-level passwords (\"%s\" "
-				"and \"%s\") for cisco \"%s\"\n",
-				pmGetProgname(), cisco[i].passwd, mypasswd, 
-				cisco[i].host);
+				"%s: conflicting user-level passwords"
+				" for cisco \"%s\"\n",
+				pmGetProgname(), cisco[i].host);
 			exit(1);
 		    }
 		}
