@@ -1184,7 +1184,7 @@ darwin_instance(pmInDom indom, int inst, char *name, pmInResult **result, pmdaEx
 }
 
 static int
-darwin_fetch(int numpmid, pmID pmidlist[], pmResult **resp, pmdaExt *pmda)
+darwin_fetch(int numpmid, pmID pmidlist[], pmdaResult **resp, pmdaExt *pmda)
 {
     int	i, need_refresh[NUM_CLUSTERS] = { 0 };
 
@@ -1238,6 +1238,7 @@ usage(void)
 {
     fprintf(stderr, "Usage: %s [options]\n\n", pmGetProgname());
     fputs("Options:\n"
+"  -D debug     set debug options, see pmdbg(1)\n"
 "  -d domain    use domain (numeric) for metrics domain of PMDA\n"
 "  -l logfile   write log into logfile rather than using default log name\n"
 "  -U username  user account to run under (default \"pcp\")\n"
