@@ -1184,7 +1184,7 @@ dm_fetch_refresh(pmdaExt *pmda, int *need_refresh)
 }
 
 static int
-dm_fetch(int numpmid, pmID pmidlist[], pmResult **resp, pmdaExt *pmda)
+dm_fetch(int numpmid, pmID pmidlist[], pmdaResult **resp, pmdaExt *pmda)
 {
     int i, sts, need_refresh[NUM_CLUSTERS] = { 0 };
 
