@@ -1,7 +1,7 @@
 /*
  * Seagate Field Accessible Reliability Metrics (FARM) Log
  *
- * Copyright (c) 2023 Red Hat.
+ * Copyright (c) 2023 - 2025 Red Hat.
  * 
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -24,13 +24,22 @@ extern pmInDom farm_indom(int);
 
 enum {
 	CLUSTER_ATA_LOG_HEADER = 0,
-	CLUSTER_ATA_DRIVE_INFORMATION,
-	CLUSTER_ATA_WORKLOAD_STATISTICS,
-	CLUSTER_ATA_ERROR_STATISTICS,
-	CLUSTER_ATA_ENVIRONMENTAL_STATISTICS,
-	CLUSTER_ATA_RELIABILITY_STATISTICS,
-	CLUSTER_ATA_LED_FLASH_EVENTS,
-	CLUSTER_ATA_PER_HEAD_STATS,
+	CLUSTER_ATA_DRIVE_INFORMATION = 1,
+	CLUSTER_ATA_WORKLOAD_STATISTICS = 2,
+	CLUSTER_ATA_ERROR_STATISTICS = 3,
+	CLUSTER_ATA_ENVIRONMENTAL_STATISTICS = 4,
+	CLUSTER_ATA_RELIABILITY_STATISTICS = 5,
+	CLUSTER_ATA_LED_FLASH_EVENTS = 6,
+	CLUSTER_ATA_PER_HEAD_STATS = 7,
+	CLUSTER_SCSI_LOG_HEADER = 10,
+	CLUSTER_SCSI_DRIVE_INFORMATION = 11,
+	CLUSTER_SCSI_WORKLOAD_STATISTICS = 12,
+	CLUSTER_SCSI_ERROR_STATISTICS = 13,
+	CLUSTER_SCSI_ENVIRONMENTAL_STATISTICS = 14,
+	CLUSTER_SCSI_RELIABILITY_STATISTICS = 15,
+	CLUSTER_SCSI_DRIVE_INFORMATION_CONTINUED = 16,
+	CLUSTER_SCSI_ENVIRONMENTAL_INFORMATION_CONTINUED = 17,
+	CLUSTER_SCSI_PER_HEAD_STATS = 18,
 	NUM_CLUSTERS
 };
 
@@ -38,11 +47,14 @@ enum {
 	DISK_INDOM = 0,
 	FLASH_LED_INDOM = 1,
 	PER_HEAD_INDOM = 2,
+	SCSI_DISK_INDOM = 3,
+	SCSI_PER_HEAD_INDOM = 4,
 	NUM_INDOMS
 };
 
 struct seagate_disk {
 	struct farm_ata_log_stats	farm_ata_log_stats;
+	struct farm_scsi_log_stats	farm_scsi_log_stats;
 };
 
 extern pmdaMetric metrictable[];
