@@ -1,7 +1,7 @@
 /*
  * Linux /proc/cpuinfo metrics cluster
  *
- * Copyright (c) 2013-2015,2017,2020-2021 Red Hat.
+ * Copyright (c) 2013-2015,2017,2020-2021,2024 Red Hat.
  * Copyright (c) 2001 Gilly Ran (gilly@exanet.com) for the
  * portions of the code supporting the Alpha platform.
  * All rights reserved.
@@ -20,6 +20,7 @@
 
 enum {
     CPUFREQ_SAMPLED	= 1<<0,
+    CPUFREQ_SCALE	= 1<<5,
     CPUFREQ_COUNT	= 1<<6,
     CPUFREQ_TIME	= 1<<7,
     CPUFREQ_MAX		= 1<<8,
@@ -29,4 +30,5 @@ enum {
 extern int refresh_proc_cpuinfo(void);
 extern int refresh_sysfs_online(char *, const char *);
 extern int refresh_sysfs_frequency_scaling(char *, int, percpu_t *);
+extern int refresh_sysfs_frequency_scaling_cur_freq(char *, int, percpu_t *);
 extern unsigned long refresh_sysfs_thermal_throttle(char *, const char *, const char *);
