@@ -459,7 +459,7 @@ do_sysctl(mib_t *mp, size_t xpect)
      * Note zero trip if mp->m_data and mp->m_datalen are already valid
      * and current
      */
-    if (pmDebugOptions.appl0 && pmDebugOptions.desperate) {
+    if (pmDebugOptions.appl0) {
 	fprintf(stderr, "do_sysctl(%s, %d) m_data=%p m_datalen=%d m_fetched=%d\n", 
 	    mp->m_name, (int)xpect, mp->m_data, (int)mp->m_datalen, mp->m_fetched);
     }
@@ -707,7 +707,7 @@ xtract:
  * then do the fetch
  */
 static int
-openbsd_fetch(int numpmid, pmID pmidlist[], pmResult **resp, pmdaExt *pmda)
+openbsd_fetch(int numpmid, pmID pmidlist[], pmdaResult **resp, pmdaExt *pmda)
 {
     int		i;
     int		done_disk = 0;
@@ -971,6 +971,7 @@ usage(void)
 {
     fprintf(stderr, "Usage: %s [options]\n\n", pmGetProgname());
     fputs("Options:\n"
+	  "  -D debug     set debug options, see pmdbg(1)\n"
 	  "  -d domain    use domain (numeric) for metrics domain of PMDA\n"
 	  "  -l logfile   write log into logfile rather than using default log name\n"
 	  "  -U username  user account to run under (default \"pcp\")\n"
