@@ -14,7 +14,7 @@
  */
 
 #include "pmapi.h"
-#include "impl.h"
+#include "libpcp.h"
 #include "trace_dev.h"
 #include "client.h"
 #include "comms.h"
@@ -40,7 +40,7 @@ acceptClient(int reqfd)
     addrlen = __pmSockAddrSize();
     fd = __pmAccept(reqfd, clients[i].addr, &addrlen);
     if (fd == -1) {
-	__pmNotifyErr(LOG_ERR, "acceptClient(%d) accept: %s",
+	pmNotifyErr(LOG_ERR, "acceptClient(%d) accept: %s",
 		reqfd, netstrerror());
 	return NULL;
     }
@@ -67,13 +67,13 @@ newClient(void)
 	clientsize = clientsize ? clientsize * 2 : MIN_CLIENTS_ALLOC;
 	clients = (client_t *) realloc(clients, sizeof(client_t)*clientsize);
 	if (clients == NULL)
-	    __pmNoMem("newClient", sizeof(client_t)*clientsize, PM_FATAL_ERR);
+	    pmNoMem("newClient", sizeof(client_t)*clientsize, PM_FATAL_ERR);
 	for (j = i; j < clientsize; j++)
 	    clients[j].addr = NULL;
     }
     clients[i].addr = __pmSockAddrAlloc();
     if (clients[i].addr == NULL)
-	__pmNoMem("newClient", __pmSockAddrSize(), PM_FATAL_ERR);
+	pmNoMem("newClient", __pmSockAddrSize(), PM_FATAL_ERR);
     if (i >= nclients)
 	nclients = i + 1;
     return i;
@@ -89,11 +89,9 @@ deleteClient(client_t *cp)
 	    break;
 
     if (i == nclients) {
-#ifdef PCP_DEBUG
-	if (pmDebug & DBG_TRACE_APPL0) {
-	    __pmNotifyErr(LOG_ERR, "deleteClient: tried to delete non-existent client");
+	if (pmDebugOptions.appl0) {
+	    pmNotifyErr(LOG_ERR, "deleteClient: tried to delete non-existent client");
 	}
-#endif
 	return;
     }
     if (cp->fd != -1) {
@@ -112,10 +110,8 @@ deleteClient(client_t *cp)
     cp->status.connected = 0;
     cp->status.padding = 0;
     cp->status.protocol = 1;	/* sync */
-#ifdef PCP_DEBUG
-	if (pmDebug & DBG_TRACE_APPL0)
-	    __pmNotifyErr(LOG_DEBUG, "deleteClient: client removed (fd=%d)", cp->fd);
-#endif
+	if (pmDebugOptions.appl0)
+	    pmNotifyErr(LOG_DEBUG, "deleteClient: client removed (fd=%d)", cp->fd);
     cp->fd = -1;
 }
 
@@ -124,18 +120,21 @@ showClients(void)
 {
     int i;
 
-    fprintf(stderr, "%s: %d connected clients:\n", pmProgname, nclients);
+    fprintf(stderr, "%s: %d connected clients:\n", pmGetProgname(), nclients);
     fprintf(stderr, "     fd  type   conn  client connection from\n"
 		    "     ==  =====  ====  ======================\n");
     for (i=0; i < nclients; i++) {
 	char *hostName;
+	char *hostAddr;
 
 	fprintf(stderr, "    %3d", clients[i].fd);
 	fprintf(stderr, "  %s  ", clients[i].status.protocol == 1 ? "sync ":"async");
 	fprintf(stderr, "%s  ", clients[i].status.connected == 1 ? "up  ":"down");
 	hostName = __pmGetNameInfo(clients[i].addr);
 	if (hostName == NULL) {
-	    fprintf(stderr, "%s", __pmSockAddrToString(clients[i].addr));
+	    hostAddr = __pmSockAddrToString(clients[i].addr);
+	    fprintf(stderr, "%s", hostAddr);
+	    free(hostAddr);
 	} else {
 	    fprintf(stderr, "%-40.40s", hostName);
 	    free(hostName);
