@@ -13,11 +13,12 @@
  */
 
 #include "pmapi.h"
-#include "impl.h"
-#include "trace_dev.h"
+#include "libpcp.h"
+#include "deprecated.h"
 #include "pmda.h"
 #include "domain.h"
 #include "data.h"
+#include "trace_dev.h"
 
 static pmdaIndom indomtab[] = {	/* list of trace metric instance domains */
 #define TRANSACT_INDOM	0
@@ -107,7 +108,7 @@ static pmdaMetric metrictab[] = {
 	PMDA_PMUNITS(0,0,0, 0,0,0)} },
 /* control.debug */
     { NULL,
-      { PMDA_PMID(0,16), PM_TYPE_32, PM_INDOM_NULL, PM_SEM_DISCRETE,
+      { PMDA_PMID(0,16), PM_TYPE_STRING, PM_INDOM_NULL, PM_SEM_DISCRETE,
 	PMDA_PMUNITS(0,0,0, 0,0,0) }, },
 /* counter.count */
     { NULL,
@@ -131,8 +132,8 @@ extern struct timeval	timespan;
 extern struct timeval	interval;
 
 static ringbuf_t	ringbuf;	/*  *THE* ring buffer of trace data */
-static __pmHashTable	summary;	/* globals + recent ringbuf summary */
-static __pmHashTable	history;	/* holds every instance seen so far */
+static hashtable_t	summary;	/* globals + recent ringbuf summary */
+static hashtable_t	history;	/* holds every instance seen so far */
 static unsigned int	rpos;		/* `working' buffer, within ringbuf */
 static unsigned int	dosummary = 1;	/* summary refreshed this interval? */
 static unsigned int	tindomsize = 0;	/*   updated local to fetch only    */
@@ -156,14 +157,14 @@ updateValueUnits(const char *str, int offset)
     for (i = 0; i < 6; i++) {
 	if ((s = strtok((i==0 ? sptr : NULL), ",")) == NULL) {
 	    fprintf(stderr, "%s: token parse error in string \"%s\"\n",
-		    pmProgname, str);
+		    pmGetProgname(), str);
 	    sts = -1;
 	    goto leaving;
 	}
 	units[i] = (int)strtol(s, &endp, 10);
 	if (*endp) {
 	    fprintf(stderr, "%s: integer parse error for substring \"%s\"\n",
-		    pmProgname, s);
+		    pmGetProgname(), s);
 	    sts = -1;
 	    goto leaving;
 	}
@@ -177,14 +178,12 @@ updateValueUnits(const char *str, int offset)
     metrictab[offset].m_desc.units.scaleTime = units[4];
     metrictab[offset].m_desc.units.scaleCount = units[5];
 
-#ifdef PCP_DEBUG
-    if (pmDebug & DBG_TRACE_APPL0) {
+    if (pmDebugOptions.appl0) {
 	fprintf(stderr, "%s: value metric units updated using \"%s\"\n"
 		"dimSpace=%d, dimTime=%d, dimCount=%d, scaleSpace=%d, "
-		"scaleTime=%d, scaleCount=%d\n", pmProgname, str,
+		"scaleTime=%d, scaleCount=%d\n", pmGetProgname(), str,
 		units[0], units[1], units[2], units[3], units[4], units[5]);
     }
-#endif
 
 leaving:
     if (sptr != NULL)
@@ -242,7 +241,7 @@ indomSortCheck(void)
  * _sorted_ contents of the instance domain.
  */
 static int
-traceInstance(pmInDom indom, int foo, char *bar, __pmInResult **iresp, pmdaExt
+traceInstance(pmInDom indom, int foo, char *bar, pmInResult **iresp, pmdaExt
  *pmda)
 {
     indomSortCheck();
@@ -263,11 +262,10 @@ summarydel(void *a)
     check.instid = k->id;
     if (__pmhashlookup(&history, check.tag, &check) == NULL) {
 	if (__pmhashinsert(&history, check.tag, &check) < 0)
-	    __pmNotifyErr(LOG_ERR, "history table insert failure - '%s' "
+	    pmNotifyErr(LOG_ERR, "history table insert failure - '%s' "
 		"instance will not maintain its instance number.", check.tag);
     }
-    if (k != NULL)
-	free(k);	/* don't free k->tag - its in the history table */
+    free(k);	/* don't free k->tag - its in the history table */
 }
 
 /*
@@ -289,7 +287,7 @@ readData(int clientfd, int *protocol)
     int			freeflag=0;
 
     if ((sts = __pmtracegetPDU(clientfd, TRACE_TIMEOUT_NEVER, &result)) < 0) {
-	__pmNotifyErr(LOG_ERR, "bogus PDU read - %s", pmtraceerrstr(sts));
+	pmNotifyErr(LOG_ERR, "bogus PDU read - %s", pmtraceerrstr(sts));
 	return -1;
     }
     else if (sts == TRACE_PDU_DATA) {
@@ -297,7 +295,7 @@ readData(int clientfd, int *protocol)
 						&type, protocol, &data)) < 0)
 	    return -1;
 	if (type < TRACE_FIRST_TYPE || type > TRACE_LAST_TYPE) {
-	    __pmNotifyErr(LOG_ERR, "unknown trace type for '%s' (%d)", tag, type);
+	    pmNotifyErr(LOG_ERR, "unknown trace type for '%s' (%d)", tag, type);
 	    free(tag);
 	    return -1;
 	}
@@ -309,8 +307,9 @@ readData(int clientfd, int *protocol)
 	return -1;
     }
     else {	/* unknown PDU type - bail & later kill connection */
-	__pmNotifyErr(LOG_ERR, "unknown PDU - expected data PDU"
+	pmNotifyErr(LOG_ERR, "unknown PDU - expected data PDU"
 		" (not type #%d)", sts);
+	__pmDumpPDUTrace(stderr);
 	return -1;
     }
 
@@ -346,13 +345,11 @@ readData(int clientfd, int *protocol)
 	newhash.txmin = newhash.txmax = data;
 	newhash.txsum = data;
 	hptr = &newhash;
-#ifdef PCP_DEBUG
-	if (pmDebug & DBG_TRACE_APPL0)
-	    __pmNotifyErr(LOG_DEBUG, "'%s' is new to the summary table!",
+	if (pmDebugOptions.appl0)
+	    pmNotifyErr(LOG_DEBUG, "'%s' is new to the summary table!",
 		    hptr->tag);
-#endif
 	if (__pmhashinsert(&summary, hptr->tag, hptr) < 0) {
-	    __pmNotifyErr(LOG_ERR, "summary table insert failure - '%s' "
+	    pmNotifyErr(LOG_ERR, "summary table insert failure - '%s' "
 		"data ignored", hptr->tag);
 	    free(hptr->tag);
 	    return -1;
@@ -384,7 +381,7 @@ readData(int clientfd, int *protocol)
 	size = (index+1)*(int)sizeof(pmdaInstid);
 	if ((indomtab[indom].it_set = (pmdaInstid *)
 			realloc(indomtab[indom].it_set, size)) == NULL) {
-	    __pmNotifyErr(LOG_ERR, "dropping instance '%s': %s", hptr->tag,
+	    pmNotifyErr(LOG_ERR, "dropping instance '%s': %s", hptr->tag,
 							osstrerror());
 	    free(hptr->tag);
 	    return -1;
@@ -392,20 +389,18 @@ readData(int clientfd, int *protocol)
 	indomtab[indom].it_set[index].i_inst = hptr->id;
 	indomtab[indom].it_set[index].i_name = hptr->tag;
 	indomtab[indom].it_numinst++;
-#ifdef PCP_DEBUG
-	if (pmDebug & DBG_TRACE_APPL0) {
+	if (pmDebugOptions.appl0) {
 	    fprintf(stderr, "Updated indom #%d:\n", indom);
 	    for (index=0; index < indomtab[indom].it_numinst; index++)
 		fprintf(stderr, "  Instance %d='%s'\n",
 			indomtab[indom].it_set[index].i_inst,
 			indomtab[indom].it_set[index].i_name);
 	}
-#endif
     }
     else {	/* update an existing entry */
 	freeflag++;	/* wont need tag afterwards, so mark for deletion */
 	if (hptr->taglength != newhash.taglength) {
-	    __pmNotifyErr(LOG_ERR, "hash table update failure - '%s' "
+	    pmNotifyErr(LOG_ERR, "hash table update failure - '%s' "
 		"data ignored (bad tag length)", tag);
 	    free(newhash.tag);
 	    return -1;
@@ -421,11 +416,9 @@ readData(int clientfd, int *protocol)
 	    else if (hptr->tracetype == TRACE_TYPE_OBSERVE)
 		hptr->txsum = data;
 		/* observations are 'permanent' and immediately available */
-#ifdef PCP_DEBUG
-	    if (pmDebug & DBG_TRACE_APPL0)
-		__pmNotifyErr(LOG_DEBUG, "'%s' real count updated (%d)",
+	    if (pmDebugOptions.appl0)
+		pmNotifyErr(LOG_DEBUG, "'%s' real count updated (%d)",
 			hptr->tag, (int)hptr->realcount);
-#endif
 	}
     }
 
@@ -446,14 +439,12 @@ readData(int clientfd, int *protocol)
 	hash.txmin = hash.txmax = data;
 	hash.txsum = data;
 	hptr = &hash;
-#ifdef PCP_DEBUG
-	if (pmDebug & DBG_TRACE_APPL0)
-	    __pmNotifyErr(LOG_DEBUG, "fresh interval data on fd=%d rpos=%d "
+	if (pmDebugOptions.appl0)
+	    pmNotifyErr(LOG_DEBUG, "fresh interval data on fd=%d rpos=%d "
 		    "('%s': len=%d type=%d value=%d)", clientfd, rpos,
 		    hash.tag, taglen, type, (int)data);
-#endif
 	if (__pmhashinsert(ringbuf.ring[rpos].stats, hash.tag, hptr) < 0) {
-	    __pmNotifyErr(LOG_ERR, "ring buffer insert failure - '%s' "
+	    pmNotifyErr(LOG_ERR, "ring buffer insert failure - '%s' "
 		"data ignored", hash.tag);
 	    free(hash.tag);
 	    return -1;
@@ -468,13 +459,11 @@ readData(int clientfd, int *protocol)
 		hptr->txmax = data;
 	    hptr->txsum += data;
 	}
-#ifdef PCP_DEBUG
-	if (pmDebug & DBG_TRACE_APPL0)
-	    __pmNotifyErr(LOG_DEBUG, "Updating data on fd=%d ('%s': type=%d "
+	if (pmDebugOptions.appl0)
+	    pmNotifyErr(LOG_DEBUG, "Updating data on fd=%d ('%s': type=%d "
 		    "count=%d min=%f max=%f sum=%f)",
 		    clientfd, hptr->tag, hptr->tracetype,
 		    hptr->txcount, hptr->txmin, hptr->txmax, hptr->txsum);
-#endif
     }
     if (freeflag) {
 	free(newhash.tag);
@@ -485,7 +474,7 @@ readData(int clientfd, int *protocol)
 }
 
 static void
-clearTable(__pmHashTable *t, void *entry)
+clearTable(hashtable_t *t, void *entry)
 {
     hashdata_t	*h = (hashdata_t *)entry;
     h->txcount = -1;	/* flag as out-of-date */
@@ -506,7 +495,7 @@ timerUpdate(void)
     }
 
     if (ringbuf.ring[rpos].working == 0) {
-	__pmNotifyErr(LOG_ERR, "buffered I/O error - ignoring timer event");
+	pmNotifyErr(LOG_ERR, "buffered I/O error - ignoring timer event");
 	return;
     }
 
@@ -518,26 +507,22 @@ timerUpdate(void)
     __pmhashtrunc(ringbuf.ring[rpos].stats);	/* new working set */
     ringbuf.ring[rpos].working = 1;
 
-#ifdef PCP_DEBUG
-    if (pmDebug & DBG_TRACE_APPL0)
-	__pmNotifyErr(LOG_DEBUG, "Alarm - working buffer is now %d/%d",
+    if (pmDebugOptions.appl0)
+	pmNotifyErr(LOG_DEBUG, "Alarm - working buffer is now %d/%d",
 		rpos+1, rbufsize);
-#endif
 }
 
 static void
-summariseDataAux(__pmHashTable *t, void *entry)
+summariseDataAux(hashtable_t *t, void *entry)
 {
     hashdata_t	*hptr;
     hashdata_t	*base = (hashdata_t *)entry;
 
-#ifdef PCP_DEBUG
-    if (pmDebug & DBG_TRACE_APPL0)
-	__pmNotifyErr(LOG_DEBUG, "summariseDataAux: looking up '%s'", base->tag);
-#endif
+    if (pmDebugOptions.appl0)
+	pmNotifyErr(LOG_DEBUG, "summariseDataAux: looking up '%s'", base->tag);
     /* update summary hash table */
     if ((hptr = __pmhashlookup(&summary, base->tag, base)) == NULL)
-	__pmNotifyErr(LOG_ERR, "summariseDataAux: entry in ring buffer, "
+	pmNotifyErr(LOG_ERR, "summariseDataAux: entry in ring buffer, "
 		"but not summary ('%s')", base->tag);
     else {	/* update an existing summary */
 	if (hptr->tracetype == TRACE_TYPE_TRANSACT) {
@@ -566,7 +551,7 @@ summariseDataAux(__pmHashTable *t, void *entry)
 		else if (hptr->tracetype == TRACE_TYPE_OBSERVE)
 		    oindomsize++;
 		else {
-		    __pmNotifyErr(LOG_ERR,
+		    pmNotifyErr(LOG_ERR,
 				"bogus trace type - skipping '%s'", hptr->tag);
 		    return;
 		}
@@ -589,10 +574,8 @@ summariseData(void)
 {
     int	count;
 
-#ifdef PCP_DEBUG
-    if (pmDebug & DBG_TRACE_APPL0)
-	__pmNotifyErr(LOG_DEBUG, "summariseData: resummarising");
-#endif
+    if (pmDebugOptions.appl0)
+	pmNotifyErr(LOG_DEBUG, "summariseData: resummarising");
     /* initialise counters */
     tindomsize = pindomsize = oindomsize = 0;
 
@@ -600,11 +583,10 @@ summariseData(void)
     for (count=0; count < rbufsize; count++) {
 	if (ringbuf.ring[count].working == 1)
 	    continue;
-#ifdef PCP_DEBUG
-    if (pmDebug & DBG_TRACE_APPL0)
-	__pmNotifyErr(LOG_DEBUG, "ring buffer table %d/%d has %d entries.\n",
-		count, rbufsize, ringbuf.ring[count].stats->entries);
-#endif
+	if (pmDebugOptions.appl0) {
+	    pmNotifyErr(LOG_DEBUG, "ring buffer table %d/%d has %d entries.\n",
+			  count, rbufsize, ringbuf.ring[count].stats->entries);
+	}
 	__pmhashtraverse(ringbuf.ring[count].stats, summariseDataAux);
     }
 
@@ -644,19 +626,21 @@ nextTraceInst(pmdaExt *pmda)
 }
 
 static int
-auxFetch(int inst, __pmID_int *idp, char *tag, pmAtomValue *atom)
+auxFetch(int inst, pmID pmid, char *tag, pmAtomValue *atom)
 {
-    if (inst != PM_IN_NULL && idp->cluster != 0)
+    static char		*last_debug = NULL;
+
+    if (inst != PM_IN_NULL && pmID_cluster(pmid) != 0)
 	return PM_ERR_INST;
 
     /* transaction, point, counter and observe trace values and control data */
-    if (idp->cluster == 0) {
+    if (pmID_cluster(pmid) == 0) {
 	hashdata_t	hash;
 	hashdata_t	*hptr;
 
 	hash.tag = tag;
 
-	switch (idp->item) {
+	switch (pmID_item(pmid)) {
 	case 0:				/* trace.transact.count */
 	    hash.tracetype = TRACE_TYPE_TRANSACT;
 	    if ((hptr = __pmhashlookup(&summary, hash.tag, &hash)) == NULL)
@@ -670,11 +654,9 @@ auxFetch(int inst, __pmID_int *idp, char *tag, pmAtomValue *atom)
 	    if (hptr->txcount < 0)	/* not in current time period */
 		return 0;
 	    atom->f = (float)((double)hptr->txcount/(double)timespan.tv_sec);
-#ifdef PCP_DEBUG
-	    if (pmDebug & DBG_TRACE_APPL1)
-		__pmNotifyErr(LOG_DEBUG, "rate=%f=%f/%f('%s')\n", (float)atom->f,
+	    if (pmDebugOptions.appl1)
+		pmNotifyErr(LOG_DEBUG, "rate=%f=%f/%f('%s')\n", (float)atom->f,
 			(double)hptr->txcount, (double)timespan.tv_sec, tag);
-#endif
 	    break;
 	case 2:				/* trace.transact.ave_time */
 	    hash.tracetype = TRACE_TYPE_TRANSACT;
@@ -686,11 +668,9 @@ auxFetch(int inst, __pmID_int *idp, char *tag, pmAtomValue *atom)
 		atom->f = 0;
 	    else {
 		atom->f = (float)((double)hptr->txsum/(double)hptr->txcount);
-#ifdef PCP_DEBUG
-		if (pmDebug & DBG_TRACE_APPL1)
-		    __pmNotifyErr(LOG_DEBUG, "ave_time=%f=%f/%f('%s')\n", (float)atom->f,
+		if (pmDebugOptions.appl1)
+		    pmNotifyErr(LOG_DEBUG, "ave_time=%f=%f/%f('%s')\n", (float)atom->f,
 			(double)hptr->txsum, (double)hptr->txcount, tag);
-#endif
 	    }
 	    break;
 	case 3:				/* trace.transact.min_time */
@@ -728,15 +708,13 @@ auxFetch(int inst, __pmID_int *idp, char *tag, pmAtomValue *atom)
 	    if (hptr->txcount < 0)	/* not in current time period */
 		return 0;
 	    atom->f = (float)((double)hptr->txcount/(double)timespan.tv_sec);
-#ifdef PCP_DEBUG
-	    if (pmDebug & DBG_TRACE_APPL1)
-		__pmNotifyErr(LOG_DEBUG, "rate=%f=%f/%f('%s')\n", (float)atom->f,
+	    if (pmDebugOptions.appl1)
+		pmNotifyErr(LOG_DEBUG, "rate=%f=%f/%f('%s')\n", (float)atom->f,
 			(double)hptr->txcount, (double)timespan.tv_sec, tag);
-#endif
 	    break;
 	case 8:				/* trace.observe.count */
 	case 17:			/* trace.counter.count */
-	    hash.tracetype = (idp->item == 8)?
+	    hash.tracetype = (pmID_item(pmid) == 8)?
 			TRACE_TYPE_OBSERVE : TRACE_TYPE_COUNTER;
 	    if ((hptr = __pmhashlookup(&summary, hash.tag, &hash)) == NULL)
 		return PM_ERR_INST;
@@ -744,22 +722,20 @@ auxFetch(int inst, __pmID_int *idp, char *tag, pmAtomValue *atom)
 	    break;
 	case 9:				/* trace.observe.rate */
 	case 18:			/* trace.counter.rate */
-	    hash.tracetype = (idp->item == 9)?
+	    hash.tracetype = (pmID_item(pmid) == 9)?
 			TRACE_TYPE_OBSERVE : TRACE_TYPE_COUNTER;
 	    if ((hptr = __pmhashlookup(&summary, hash.tag, &hash)) == NULL)
 		return PM_ERR_INST;
 	    if (hptr->txcount < 0)	/* not in current time period */
 		return 0;
 	    atom->f = (float)((double)hptr->txcount/(double)timespan.tv_sec);
-#ifdef PCP_DEBUG
-	    if (pmDebug & DBG_TRACE_APPL1)
-		__pmNotifyErr(LOG_DEBUG, "rate=%f=%f/%f('%s')\n", (float)atom->f,
+	    if (pmDebugOptions.appl1)
+		pmNotifyErr(LOG_DEBUG, "rate=%f=%f/%f('%s')\n", (float)atom->f,
 			(double)hptr->txcount, (double)timespan.tv_sec, tag);
-#endif
 	    break;
 	case 10:			/* trace.observe.value */
 	case 19:			/* trace.counter.value */
-	    hash.tracetype = (idp->item == 10)?
+	    hash.tracetype = (pmID_item(pmid) == 10)?
 			TRACE_TYPE_OBSERVE : TRACE_TYPE_COUNTER;
 	    if ((hptr = __pmhashlookup(&summary, hash.tag, &hash)) == NULL)
 		return PM_ERR_INST;
@@ -781,7 +757,10 @@ auxFetch(int inst, __pmID_int *idp, char *tag, pmAtomValue *atom)
 	    atom->ul = 1;
 	    break;
 	case 16:			/* trace.control.debug */
-	    atom->ul = pmDebug;
+	    if (last_debug != NULL)
+		free(last_debug);
+	    last_debug = pmGetDebug();
+	    atom->cp = last_debug;
 	    break;
 	default:
 	    return PM_ERR_PMID;
@@ -794,14 +773,14 @@ auxFetch(int inst, __pmID_int *idp, char *tag, pmAtomValue *atom)
 }
 
 static int
-getIndomSize(__pmID_int *pmidp)
+getIndomSize(pmID pmid)
 {
     int size;
 
-    if (pmidp->cluster != 0)
+    if (pmID_cluster(pmid) != 0)
 	return 1;
 
-    switch (pmidp->item) {
+    switch (pmID_item(pmid)) {
 	case 0:		/* uses summary's real counters (transact) */
 	case 5:
 	    size = indomtab[TRANSACT_INDOM].it_numinst;
@@ -840,13 +819,12 @@ getIndomSize(__pmID_int *pmidp)
 
 
 static int
-traceFetch(int numpmid, pmID pmidlist[], pmResult **resp, pmdaExt *pmda)
+traceFetch(int numpmid, pmID pmidlist[], pmdaResult **resp, pmdaExt *pmda)
 {
     static int		maxnpmids = 0;
-    static pmResult	*res = NULL;
+    static pmdaResult	*res = NULL;
     pmValueSet		*vset;
     pmDesc		*dp;
-    __pmID_int		*pmidp;
     pmdaMetric		*metap;
     pmAtomValue		atom;
     pmdaInstid		*ins;
@@ -860,9 +838,9 @@ traceFetch(int numpmid, pmID pmidlist[], pmResult **resp, pmdaExt *pmda)
     if (numpmid > maxnpmids) {
 	if (res != NULL)
 	    free(res);
-	/* (numpmid - 1) because there's room for one valueSet in a pmResult */
-	need = (int)sizeof(pmResult) + (numpmid-1)*(int)sizeof(pmValueSet *);
-	if ((res = (pmResult *) malloc(need)) == NULL) {
+	/* (numpmid - 1) because there's room for one valueSet in a pmdaResult */
+	need = (int)sizeof(pmdaResult) + (numpmid-1)*(int)sizeof(pmValueSet *);
+	if ((res = (pmdaResult *) malloc(need)) == NULL) {
 	    return -oserror();
 	}
 	maxnpmids = numpmid;
@@ -875,11 +853,10 @@ traceFetch(int numpmid, pmID pmidlist[], pmResult **resp, pmdaExt *pmda)
     for (i = 0; i < numpmid; i++) {
 	dp = NULL;
 	metap = NULL;
-	pmidp = (__pmID_int *)&pmidlist[i];
 	if (pmda->e_direct) {
-	    if (pmidp->item < pmda->e_nmetrics &&
-		pmidlist[i] == pmda->e_metrics[pmidp->item].m_desc.pmid) {
-		metap = &pmda->e_metrics[pmidp->item];
+	    if (pmID_item(pmidlist[i]) < pmda->e_nmetrics &&
+		pmidlist[i] == pmda->e_metrics[pmID_item(pmidlist[i])].m_desc.pmid) {
+		metap = &pmda->e_metrics[pmID_item(pmidlist[i])];
 		dp = &(metap->m_desc);
 	    }
 	}
@@ -893,7 +870,7 @@ traceFetch(int numpmid, pmID pmidlist[], pmResult **resp, pmdaExt *pmda)
 	    }
 	}
 	if (dp == NULL) {
-	    __pmNotifyErr(LOG_ERR, "traceFetch: Requested metric %s is not "
+	    pmNotifyErr(LOG_ERR, "traceFetch: Requested metric %s is not "
 					"defined", pmIDStr(pmidlist[i]));
 	    numval = PM_ERR_PMID;
 	}
@@ -904,12 +881,10 @@ traceFetch(int numpmid, pmID pmidlist[], pmResult **resp, pmdaExt *pmda)
 	     */
 	    if (dosummary == 1)
 		summariseData();
-	    numval = getIndomSize(pmidp);
-#ifdef PCP_DEBUG
-    if (pmDebug & DBG_TRACE_APPL0)
-	__pmNotifyErr(LOG_DEBUG, "instance domain for %s numval=%d",
+	    numval = getIndomSize(pmidlist[i]);
+    if (pmDebugOptions.appl0)
+	pmNotifyErr(LOG_DEBUG, "instance domain for %s numval=%d",
 		pmIDStr(dp->pmid), numval);
-#endif
 	}
 	else
 	    numval = 1;
@@ -923,7 +898,7 @@ traceFetch(int numpmid, pmID pmidlist[], pmResult **resp, pmdaExt *pmda)
 					sizeof(pmValue));
 	if (vset == NULL) {
 	    if ((res->numpmid = i) > 0)
-		__pmFreeResultValues(res);
+		pmdaFreeResultValues(res);
 	    return -oserror();
 	}
 
@@ -942,10 +917,10 @@ traceFetch(int numpmid, pmID pmidlist[], pmResult **resp, pmdaExt *pmda)
 	j = 0;
 	do {
 	    if (ins == NULL) {
-		__pmNotifyErr(LOG_ERR, "bogus instance ignored (pmid=%s)",
+		pmNotifyErr(LOG_ERR, "bogus instance ignored (pmid=%s)",
 					pmIDStr(dp->pmid));
 		if ((res->numpmid = i) > 0)
-		    __pmFreeResultValues(res);
+		    pmdaFreeResultValues(res);
 		return PM_ERR_INST;
 	    }
 	    if (j == numval) {
@@ -954,40 +929,36 @@ traceFetch(int numpmid, pmID pmidlist[], pmResult **resp, pmdaExt *pmda)
 			sizeof(pmValueSet) + (numval - 1)*sizeof(pmValue));
 		if (vset == NULL) {
 		    if ((res->numpmid = i) > 0)
-			__pmFreeResultValues(res);
+			pmdaFreeResultValues(res);
 		    return -oserror();
 		}
 	    }
 	    vset->vlist[j].inst = ins->i_inst;
-	    if ((sts = auxFetch(ins->i_inst, pmidp, ins->i_name, &atom)) < 0) {
+	    if ((sts = auxFetch(ins->i_inst, pmidlist[i], ins->i_name, &atom)) < 0) {
 		if (sts == PM_ERR_PMID)
-		    __pmNotifyErr(LOG_ERR, "unknown PMID requested - '%s'",
+		    pmNotifyErr(LOG_ERR, "unknown PMID requested - '%s'",
 				pmIDStr(dp->pmid));
 		else if (sts == PM_ERR_INST)
-		    __pmNotifyErr(LOG_ERR, "unknown instance requested - %d "
+		    pmNotifyErr(LOG_ERR, "unknown instance requested - %d "
 			"(pmid=%s)", ins->i_inst, pmIDStr(dp->pmid));
 		else
-		    __pmNotifyErr(LOG_ERR, "fetch error (pmid=%s): %s",
+		    pmNotifyErr(LOG_ERR, "fetch error (pmid=%s): %s",
 				pmIDStr(dp->pmid), pmErrStr(sts));
 	    }
 	    else if (sts == 0) {	/* not current, so don't use */
-#ifdef PCP_DEBUG
-		if (pmDebug & DBG_TRACE_APPL0)
-		    __pmNotifyErr(LOG_DEBUG, "Instance is dated %s (pmid=%s)",
+		if (pmDebugOptions.appl0)
+		    pmNotifyErr(LOG_DEBUG, "Instance is dated %s (pmid=%s)",
 					ins->i_name, pmIDStr(dp->pmid));
-#endif
 	    }
 	    else if ((sts = __pmStuffValue(&atom, &vset->vlist[j], dp->type)) == PM_ERR_TYPE)
-		__pmNotifyErr(LOG_ERR, "bad desc type (%d) for metric %s",
+		pmNotifyErr(LOG_ERR, "bad desc type (%d) for metric %s",
 				dp->type, pmIDStr(dp->pmid));
 	    else if (sts >= 0) {
 		vset->valfmt = sts;
 		j++;
-#ifdef PCP_DEBUG
-		if (pmDebug & DBG_TRACE_APPL0)
-		    __pmNotifyErr(LOG_DEBUG, "Instance is good! %s (pmid=%s)",
+		if (pmDebugOptions.appl0)
+		    pmNotifyErr(LOG_DEBUG, "Instance is good! %s (pmid=%s)",
 					ins->i_name, pmIDStr(dp->pmid));
-#endif
 	    }
 	} while (dp->indom != PM_INDOM_NULL &&
 				(ins = nextTraceInst(pmda)) != NULL);
@@ -1003,33 +974,28 @@ traceFetch(int numpmid, pmID pmidlist[], pmResult **resp, pmdaExt *pmda)
 
 
 static int
-traceStore(pmResult *result, pmdaExt *pmda)
+traceStore(pmdaResult *result, pmdaExt *pmda)
 {
     int		i, j;
     int         sts = 0;
     pmValueSet  *vsp = NULL;
-    __pmID_int   *pmidp = NULL;
-    pmAtomValue	av;
     extern int	afid;
     extern void alarming(int, void *);
 
     for (i = 0; i < result->numpmid; i++) {
 	vsp = result->vset[i];
-	pmidp = (__pmID_int *)&vsp->pmid;
 
-	if (pmidp->cluster != 0)
+	if (pmID_cluster(vsp->pmid) != 0)
 	    return PM_ERR_PMID;
 
-	if (pmidp->item == 15) {	/* trace.control.reset */
-#ifdef PCP_DEBUG
-	    if (pmDebug & DBG_TRACE_APPL0)
-		__pmNotifyErr(LOG_DEBUG, "resetting trace metrics");
-#endif
+	if (pmID_item(vsp->pmid) == 15) {	/* trace.control.reset */
+	    if (pmDebugOptions.appl0)
+		pmNotifyErr(LOG_DEBUG, "resetting trace metrics");
 	    /* reset the interval timer */
 	    if (afid >= 0) {
 		__pmAFunregister(afid);
 		if ((afid = __pmAFregister(&interval, NULL, alarming)) < 0) {
-		    __pmNotifyErr(LOG_ERR, "__pmAFregister failed");
+		    pmNotifyErr(LOG_ERR, "__pmAFregister failed");
 		    exit(1);
 		}
 	    }
@@ -1065,17 +1031,16 @@ traceStore(pmResult *result, pmdaExt *pmda)
 	    tindomsize = pindomsize = oindomsize = 0;
 	    /* definately need to recompute the summary next fetch */
 	    dosummary = 1;
-	    __pmNotifyErr(LOG_INFO, "PMDA reset");
+	    pmNotifyErr(LOG_INFO, "PMDA reset");
 	}
-	else if (pmidp->item == 16) {	/* trace.control.debug */
-	    if (vsp->numval != 1 || vsp->valfmt != PM_VAL_INSITU)
-		sts = PM_ERR_CONV;
-	    else if (sts >= 0 && ((sts = pmExtractValue(vsp->valfmt,
-			&vsp->vlist[0], PM_TYPE_32, &av, PM_TYPE_32)) >= 0)) {
-		if (pmDebug != av.l) {
-		    pmDebug = av.l;
-		    __pmNotifyErr(LOG_INFO, "debug level set to %d", pmDebug);
-		    debuglibrary(pmDebug);
+	else if (pmID_item(vsp->pmid) == 16) {	/* trace.control.debug */
+	    if (vsp->numval != 1 || vsp->valfmt == PM_VAL_INSITU)
+		sts = PM_ERR_BADSTORE;
+	    else {
+		pmClearDebug("all");
+		if ((sts = pmSetDebug(vsp->vlist[0].value.pval->vbuf)) == 0) {
+		    pmNotifyErr(LOG_INFO, "debug options set to %s", vsp->vlist[0].value.pval->vbuf);
+		    debuglibrary();
 		}
 	    }
 	}
@@ -1107,20 +1072,20 @@ traceInit(pmdaInterface *dp)
     /* initialise ring buffer */
     rsize = (int)(sizeof(statlist_t) * rbufsize);
     if ((ringbuf.ring = (statlist_t *)malloc(rsize)) == NULL) {
-	__pmNotifyErr(LOG_ERR, "failed during ring buffer initialise: %s",
+	pmNotifyErr(LOG_ERR, "failed during ring buffer initialise: %s",
 		osstrerror());
 	exit(1);
     }
     for (rsize=0; rsize < rbufsize; rsize++) {
-	if ((ringbuf.ring[rsize].stats = (__pmHashTable *)
-	 			malloc(sizeof(__pmHashTable))) == NULL) {
-	    __pmNotifyErr(LOG_ERR, "ring buffer initialise failed: %s",
+	if ((ringbuf.ring[rsize].stats = (hashtable_t *)
+	 			malloc(sizeof(hashtable_t))) == NULL) {
+	    pmNotifyErr(LOG_ERR, "ring buffer initialise failed: %s",
 		osstrerror());
 	    exit(1);
 	}
 	if ((sts = __pmhashinit(ringbuf.ring[rsize].stats, 0, sizeof(hashdata_t),
 						datacmp, datadel)) < 0) {
-	    __pmNotifyErr(LOG_ERR, "ring buffer initialisation failed: %s",
+	    pmNotifyErr(LOG_ERR, "ring buffer initialisation failed: %s",
 		osstrerror());
 	    exit(1);
 	}
@@ -1137,14 +1102,14 @@ traceInit(pmdaInterface *dp)
     dp->version.two.ext->e_idp = indomtab;
     if ((sts = __pmhashinit(&summary, 0, sizeof(hashdata_t),
 						datacmp, summarydel)) < 0) {
-	__pmNotifyErr(LOG_ERR, "summary table initialisation failed: %s",
+	pmNotifyErr(LOG_ERR, "summary table initialisation failed: %s",
 						osstrerror());
 	exit(1);
     }
     /* initialise list of reserved instance domains (for store recovery) */
     if ((sts = __pmhashinit(&history, 0, sizeof(instdata_t),
 						instcmp, instdel)) < 0) {
-	__pmNotifyErr(LOG_ERR, "history table initialisation failed: %s",
+	pmNotifyErr(LOG_ERR, "history table initialisation failed: %s",
 						osstrerror());
 	exit(1);
     }
