@@ -145,14 +145,14 @@ txmon_fetchCallBack(pmdaMetric *mdesc, unsigned int inst, pmAtomValue *atom)
 	}
     }
 
-    return 0;
+    return PMDA_FETCH_STATIC;
 }
 
 /*
  * support the storage of a value into the control metrics
  */
 static int
-txmon_store(pmResult *result, pmdaExt *pmda)
+txmon_store(pmdaResult *result, pmdaExt *pmda)
 {
     int		i;
     int		n;
@@ -215,7 +215,7 @@ txmon_init(pmdaInterface *dp)
 
     pmSetProcessIdentity(username);
 
-    dp->version.two.store = txmon_store;
+    dp->version.seven.store = txmon_store;
 
     pmdaSetFetchCallBack(dp, txmon_fetchCallBack);
     if (pmDebugOptions.appl0) {
@@ -274,7 +274,7 @@ main(int argc, char **argv)
 
     pmsprintf(mypath, sizeof(mypath), "%s%c" "txmon" "%c" "help",
 		pmGetConfig("PCP_PMDAS_DIR"), sep, sep);
-    pmdaDaemon(&dispatch, PMDA_INTERFACE_2, pmGetProgname(), TXMON,
+    pmdaDaemon(&dispatch, PMDA_INTERFACE_7, pmGetProgname(), TXMON,
 		"txmon.log", mypath);
 
     pmdaGetOptions(argc, argv, &opts, &dispatch);
@@ -345,8 +345,7 @@ main(int argc, char **argv)
 	 */
 	control->index[n] = p - (char *)control;
 	sp = (stat_t *)p;
-	strncpy(sp->type, argv[opts.optind++], MAXNAMESIZE);
-	sp->type[MAXNAMESIZE-1] = '\0';		/* ensure null terminated */
+	pmstrncpy(sp->type, MAXNAMESIZE, argv[opts.optind++]);
 	sp->reset_count = 0;
 	sp->count = 0;
 	sp->max_time = -1.0;
