@@ -12,6 +12,12 @@
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  * for more details.
+ *
+ * Debugging options
+ * -Dappl0	dump each pmResult processed
+ * -Dappl1	report each time a new metric-instance pair is seen
+ * -Dappl2	report counter values as part of wrap detection
+ * -Dappl3	report elapsed and cpu time for each pass
  */
 
 #include <limits.h>
@@ -76,15 +82,15 @@ dumpLabel(void)
 	exit(EXIT_FAILURE);
     }
 
-    fprintf(stderr, "Log Label (Log Format Version %d)\n", label.ll_magic & 0xff);
-    fprintf(stderr, "Performance metrics from host %s\n", label.ll_hostname);
+    fprintf(stderr, "Log Label (Log Format Version %d)\n", label.magic & 0xff);
+    fprintf(stderr, "Performance metrics from host %s\n", label.hostname);
 
-    time = label.ll_start.tv_sec;
+    time = label.start.tv_sec;
     ddmm = pmCtime(&time, timebuf);
     ddmm[10] = '\0';
     yr = &ddmm[20];
     fprintf(stderr, "  commencing %s ", ddmm);
-    pmPrintStamp(stderr, &label.ll_start);
+    pmtimespecPrint(stderr, &label.start);
     fprintf(stderr, " %4.4s\n", yr);
 
     if (opts.finish.tv_sec == PM_MAX_TIME_T) {
@@ -97,7 +103,7 @@ dumpLabel(void)
         ddmm[10] = '\0';
         yr = &ddmm[20];
         fprintf(stderr, "  ending     %s ", ddmm);
-        pmPrintStamp(stderr, &opts.finish);
+        pmtimespecPrint(stderr, &opts.finish);
         fprintf(stderr, " %4.4s\n", yr);
     }
 }
@@ -127,8 +133,7 @@ filter(const_dirent *dp)
      * __pmLogBaseName will strip the suffix by modifying the data
      * in place. The suffix can still be found after the base name.
      */
-    strncpy(logBase, dp->d_name, sizeof(logBase));
-    logBase[sizeof(logBase)-1] = '\0';
+    pmstrncpy(logBase, sizeof(logBase), dp->d_name);
     if (__pmLogBaseName(logBase) == NULL ) {
 	if (vflag > 2)
 	    fprintf(stderr, "no (not expected extension after basename)\n");
@@ -272,7 +277,7 @@ doit(void)
 	char	path[MAXPATHLEN];
 	if (strcmp(archdirname, ".") == 0) {
 	    /* skip ./ prefix */
-	    strncpy(path, namelist[i]->d_name, sizeof(path));
+	    pmstrncpy(path, sizeof(path), namelist[i]->d_name);
 	}
 	else {
 	    pmsprintf(path, sizeof(path), "%s%c%s", archdirname, sep, namelist[i]->d_name);
@@ -349,7 +354,7 @@ doit(void)
 
     if (strcmp(archdirname, ".") == 0)
 	/* skip ./ prefix */
-	strncpy(archname, archbasename, sizeof(archname) - 1);
+	pmstrncpy(archname, sizeof(archname), archbasename);
     else
 	pmsprintf(archname, sizeof(archname), "%s%c%s", archdirname, sep, archbasename);
 
