@@ -132,6 +132,6 @@ extern int	xarg;
  * parse -v arg
  */
 extern int
-ParseSize(char *, int *, __int64_t *, struct timeval *);
+ParseSize(char *, int *, __int64_t *, struct timespec *);
 
 #endif /* PCP_LOGGER_H */
