@@ -16,12 +16,15 @@
  * for more details.
  *
  * Debug flags
- * appl0	I/O
- * appl1	metdata changes
- * appl2	pmResult changes
- * appl3	-q and reason for not taking quick exit
+ * appl0	archive reads and writes for data and metadata
+ * appl1	metadata changes (metric descriptors, instance domains,
+ *              help text, metric labels)
+ * appl2	metric value (pmResult) changes
+ * appl3	-q handling and explanation of required changes that
+ *		are the reason for not taking a quick exit
  * appl4	config parser
  * appl5	regexp matching for metric value changes and iname changes
+ * appl6	lexical scanner
  */
 
 #include <math.h>
@@ -1691,10 +1694,8 @@ main(int argc, char **argv)
 	int	sep = pmPathSeparator();
 
 #if HAVE_MKSTEMP
-	strncpy(path, argv[argc-1], sizeof(path));
-	path[sizeof(path)-1] = '\0';
-	strncpy(dname, dirname(path), sizeof(dname));
-	dname[sizeof(dname)-1] = '\0';
+	pmstrncpy(path, sizeof(path), argv[argc-1]);
+	pmstrncpy(dname, sizeof(dname), dirname(path));
 	if ((dir_fd = open(dname, O_RDONLY)) < 0) {
 	    fprintf(stderr, "%s: Error: cannot open directory \"%s\" for reading: %s\n", pmGetProgname(), dname, strerror(errno));
 	    abandon();
@@ -1718,12 +1719,9 @@ main(int argc, char **argv)
 	char	fname[MAXPATHLEN+1];
 	char	*s;
 
-	strncpy(path, argv[argc-1], sizeof(path));
-	path[sizeof(path)-1] = '\0';
-	strncpy(fname, basename(path), sizeof(fname));
-	fname[sizeof(fname)-1] = '\0';
-	strncpy(dname, dirname(path), sizeof(dname));
-	dname[sizeof(dname)-1] = '\0';
+	pmstrncpy(path, sizeof(path), argv[argc-1]);
+	pmstrncpy(fname, sizeof(fname), basename(path));
+	pmstrncpy(dname, sizeof(dname), dirname(path));
 
 	if ((s = tempnam(dname, fname)) == NULL) {
 	    fprintf(stderr, "%s: Error: first tempnam() failed: %s\n", pmGetProgname(), strerror(errno));
@@ -1816,7 +1814,7 @@ main(int argc, char **argv)
      * ... and start at the same initial volume (volume 0, 1, ...
      * may be missing)
      */
-    outarch.archctl.ac_log = &outarch.logctl;
+    __pmLogWriterInit(&outarch.archctl, &outarch.logctl);
     if ((sts = __pmLogCreate("", outarch.name, outarch.version, &outarch.archctl, inarch.ctxp->c_archctl->ac_curvol)) < 0) {
 	fprintf(stderr, "%s: Error: __pmLogCreate(%s,v%d): %s\n",
 		pmGetProgname(), outarch.name, outarch.version, pmErrStr(sts));
