@@ -480,6 +480,13 @@ retype(int i, metricspec_t *mp)
 	    abandon();
 	    /*NOTREACHED*/
 	}
+	if (mp->new_desc.type == PM_TYPE_STRING) {
+	    /*
+	     * this is the only output type that is valid for pmlogrewrite
+	     * and has space allocated in pmExtractValue()
+	     */
+	    free(val.cp);
+	}
     }
     inarch.rp->vset[i]->valfmt = sts;
 }
@@ -780,7 +787,7 @@ do_result(void)
 	    fprintf(stderr, "Log: write ");
 	    stamp.tv_sec = inarch.rp->timestamp.sec;
 	    stamp.tv_usec = inarch.rp->timestamp.nsec / 1000;
-	    pmPrintStamp(stderr, &stamp);
+	    pmtimevalPrint(stderr, &stamp);
 	    fprintf(stderr, " numpmid=%d @ offset=%ld\n", inarch.rp->numpmid, out_offset);
 	}
     }
