@@ -348,6 +348,7 @@ main(int argc, char *argv[])
     int		timeseries;
     char	*envstr;
     pid_t	mainpid;
+    struct timeval	finish;
 
     umask(022);
     set_rlimit_maxfiles();
@@ -376,7 +377,7 @@ main(int argc, char *argv[])
 	 * the logfile name, just before the last ., so pmproxy.log
 	 * will become pmproxy.<pid>.log
 	 */
-	char	newlogfile[MAXPATHLEN];
+	static char	newlogfile[MAXPATHLEN];
 	char	pbuf[11];	/* enough for a 32-bit pid */
 	char	*pend = NULL;
 
@@ -384,8 +385,8 @@ main(int argc, char *argv[])
 	pend = rindex(logfile, '.');
 	if (pend == NULL) {
 	    /* no '.', so append .<pid> */
-	    strncpy(newlogfile, logfile, MAXPATHLEN-1);
-	    strcat(newlogfile, pbuf);
+	    pmstrncpy(newlogfile, MAXPATHLEN, logfile);
+	    pmstrncat(newlogfile, MAXPATHLEN, pbuf);
 	}
 	else {
 	    /* stitch name together ... <pre>.<post> -> <pre>.<pid>.<post> */
@@ -402,6 +403,7 @@ main(int argc, char *argv[])
 	    *q = '\0';
 	}
 	pmOpenLog(pmGetProgname(), newlogfile, stderr, &sts);
+	logfile = newlogfile;
     }
     else
 	pmOpenLog(pmGetProgname(), logfile, stderr, &sts);
@@ -462,7 +464,11 @@ main(int argc, char *argv[])
     fflush(stderr);
 
     /* Loop processing client connections and server responses */
-    server->loop(info, opts.finish_optarg? &opts.finish : NULL);
+    if (opts.finish_optarg) {
+	finish.tv_sec = opts.finish.tv_sec;
+	finish.tv_usec = opts.finish.tv_nsec / 1000;
+    }
+    server->loop(info, opts.finish_optarg? &finish : NULL);
 
     /* inform service manager and shutdown cleanly */
     __pmServerNotifyServiceManagerStopping(mainpid);
