@@ -1,7 +1,7 @@
 /*
- * Copyright (c) 2012-2013,2018-2019 Red Hat.
+ * Copyright (c) 2012-2013,2018-2019,2022 Red Hat.
  * Copyright (c) 2002 Silicon Graphics, Inc.  All Rights Reserved.
- * 
+ *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU Lesser General Public License as published
  * by the Free Software Foundation; either version 2.1 of the License, or
@@ -18,7 +18,7 @@
 typedef void *(*proxyOpenRequestPorts)(char *, size_t, int);
 typedef void (*proxyDumpRequestPorts)(FILE *, void *);
 typedef void (*proxyShutdownPorts)(void *);
-typedef void (*proxyMainLoop)(void *);
+typedef void (*proxyMainLoop)(void *, struct timeval *);
 
 typedef struct pmproxy {
     proxyOpenRequestPorts	openports;
