@@ -39,7 +39,6 @@ static pmLongOptions longopts[] = {
 
 static int override(int, pmOptions *);
 static pmOptions opts = {
-    .version = PMAPI_VERSION_3,
     .flags = PM_OPTFLAG_DONE | PM_OPTFLAG_BOUNDARIES | PM_OPTFLAG_STDOUT_TZ,
     .short_options = PMAPI_OPTIONS "df:i:K:LrU:vw:x:X",
     .long_options = longopts,
@@ -265,19 +264,19 @@ mytimestamp(struct timespec *stamp)
 /* Fetch metric values. */
 static int
 getvals(Context *x,		/* in - full pm description */
-        pmHighResResult **vs)		/* alloc - pm values */
+        pmResult **vs)		/* alloc - pm values */
 {
-    pmHighResResult	*r;
-    int			e;
-    int			i;
+    pmResult	*r;
+    int		e;
+    int		i;
 
     if (rawArchive) {
 	/*
-	 * for -U mode, read until we find either a pmHighResResult with the
+	 * for -U mode, read until we find either a pmResult with the
 	 * pmid we are after, or a mark record
 	 */
 	for ( ; ; ) {
-	    e = pmFetchHighResArchive(&r);
+	    e = pmFetchArchive(&r);
 	    if (e < 0)
 		break;
 
@@ -286,7 +285,7 @@ getvals(Context *x,		/* in - full pm description */
 		    mytimestamp(&r->timestamp);
 		printf("  Archive logging suspended\n");
 		reporting = 0;
-		pmFreeHighResResult(r);
+		pmFreeResult(r);
 		return -1;
 	    }
 
@@ -296,11 +295,11 @@ getvals(Context *x,		/* in - full pm description */
 	    }
 	    if (i != r->numpmid)
 		break;
-	    pmFreeHighResResult(r);
+	    pmFreeResult(r);
 	}
     }
     else {
-	e = pmFetchHighRes(1, &(x->pmid), &r);
+	e = pmFetch(1, &(x->pmid), &r);
 	i = 0;
     }
 
@@ -323,7 +322,7 @@ getvals(Context *x,		/* in - full pm description */
 	pmTimeStateAck(&controls, pmtime);
 
     if (pmtimespecToReal(&r->timestamp) > pmtimespecToReal(&opts.finish)) {
-	pmFreeHighResResult(r);
+	pmFreeResult(r);
 	return -2;
     }
 
@@ -337,12 +336,12 @@ getvals(Context *x,		/* in - full pm description */
 	    printf("No values available\n");
 	else if (rawEvents && verbose)
 	    printf("%s: No values available\n", x->metric);
-	pmFreeHighResResult(r);
+	pmFreeResult(r);
 	return -1;
     }
     else if (e < 0) {
 	if (rawEvents && e == PM_ERR_NOTCONN) {
-	    pmFreeHighResResult(r);
+	    pmFreeResult(r);
 	    exit(EXIT_SUCCESS);
 	}
 	else if (rawArchive) {
@@ -352,7 +351,7 @@ getvals(Context *x,		/* in - full pm description */
 	    fprintf(stderr, "\n%s: pmFetch: %s\n",
 			pmGetProgname(), pmErrStr(r->vset[i]->numval));
 	}
-	pmFreeHighResResult(r);
+	pmFreeResult(r);
 	return -1;
     }
 
@@ -412,11 +411,25 @@ printhdr(Context *x)
 	time_t		time;
 	printf("archive:   %s\n", source);
 	printf("host:      %s\n", x->hostname);
-	time = opts.origin.tv_sec;
-	printf("start:     %s", pmCtime(&time, tbfr));
+	printf("start:     ");
+	if (Xflag == 0) {
+	    time = opts.origin.tv_sec;
+	    printf("%s", pmCtime(&time, tbfr));
+	}
+	else {
+	    mytimestamp(&opts.origin);
+	    putchar('\n');
+	}
 	if (opts.finish.tv_sec != PM_MAX_TIME_T) {
-	    time = opts.finish.tv_sec;
-	    printf("end:       %s", pmCtime(&time, tbfr));
+	    printf("end:       ");
+	    if (Xflag == 0) {
+		time = opts.finish.tv_sec;
+		printf("%s", pmCtime(&time, tbfr));
+	    }
+	    else {
+		mytimestamp(&opts.finish);
+		putchar('\n');
+	    }
 	}
     }
 
@@ -964,8 +977,8 @@ main(int argc, char *argv[])
     char        *endnum;
     char        *errmsg;
     pmMetricSpec *msp = NULL;
-    pmHighResResult *rslt1;		/* current values */
-    pmHighResResult *rslt2 = NULL;	/* previous values */
+    pmResult 	*rslt1;		/* current values */
+    pmResult 	*rslt2 = NULL;	/* previous values */
 
     setlinebuf(stdout);
     context.iall = 1;
@@ -1362,14 +1375,14 @@ main(int argc, char *argv[])
 	 * discard previous and save current result, so this value
 	 * becomes the previous value at the next iteration
 	 */
-	pmFreeHighResResult(rslt2);
+	pmFreeResult(rslt2);
 	rslt2 = rslt1;
 	idx2 = idx1;
     }
 
     /* make valgrind happy */
     if (rslt2 != NULL)
-	pmFreeHighResResult(rslt2);
+	pmFreeResult(rslt2);
     if (msp != NULL)
 	pmFreeMetricSpec(msp);
 
